/**
 * Copyright (c) InMon Corp. ALL RIGHTS RESERVED  
 */

(function($) {
    
    /**
     * $.jqplot.AxisRangeSelector
     * Plugin that allows rubber banding to select a region of the plot
     * (represented by a range on the x-axis and/or y-axis) and generates an
     * event containing the indices of the data points of the selected range on
     * the chosen axis.
     * Example usage:
     * plot2 = $.jqplot("chart2", [s1, s2], {
     *      axisRangeSelector: {
     *          show: true,
     *          style: 'crosshair', //css cursor style; default 'crosshair'
     *          constrainTo: 'x' //denoting which axis range is to be selected; default 'x'; other vals 'y', 'none'        
     *      }
     * });
     * plot2.target.on("jqplotRangeSelected", function(evt, startIdx, endIdx) {
     *          //handleRangeSelected(startIdx, endIdx);
     *          console.log("range selected start="+startIdx+" end="+endIdx);
     */
    
    $.jqplot.AxisRangeSelector = function(options) {
        // prop: style
        // CSS spec for cursor style
        this.style = 'crosshair';
        this.previousCursor = 'auto';
        // prop: show
        // whether to enable the range selector or not.
        this.show = $.jqplot.config.enablePlugins;
        // prop: constrainTo
        // 'none', 'x' or 'y' denoting which axis the range is to be selected
        // from
        this.constrainTo = 'x';
        
        this._selection = {
                start:[], //start gridpos [x,y]
                end:[], //latest gridpos [x,y]
                started: false, 
                dragging: false, 
                axes: {start:{}, end:{}}, 
                gridpos:{}, //latest grid position
                datapos:{}}; //latest data position for all axis
        this.selectionCanvas = null;
        this._oldHandlers = {
                onselectstart: null, 
                ondrag: null, 
                onmousedown: null};
        //true if the mouse is over the grid, false if not.
        this.onGrid = false;
        $.extend(true, this, options);
    };
        
    /**
     * Called with context of plot
     * @param target
     * @param data
     * @param opts
     */
    $.jqplot.AxisRangeSelector.init = function(target, data, opts) {
        var options = opts || {};
        this.plugins.axisRangeSelector = 
            new $.jqplot.AxisRangeSelector(options.axisRangeSelector);
        var ars = this.plugins.axisRangeSelector;

        if (ars.show) {
            this.eventListenerHooks.add('jqplotMouseEnter', handleMouseEnter);
            this.eventListenerHooks.add('jqplotMouseLeave', handleMouseLeave);
            this.eventListenerHooks.add('jqplotMouseDown', handleMouseDown);
        }
    };
    
    // called with context of plot
    $.jqplot.AxisRangeSelector.postDraw = function() {
        var ars = this.plugins.axisRangeSelector;
        
        // Memory Leaks patch
        if (ars.selectionCanvas) {
            ars.selectionCanvas.resetCanvas();
            ars.selectionCanvas = null;
        }
        ars.selectionCanvas = new $.jqplot.GenericCanvas();
        this.eventCanvas._elem.before(ars.selectionCanvas.createElement(this._gridPadding, 
                'jqplot-ars-canvas', this._plotDimensions, this));
        ars.selectionCanvas.setContext();
    };
    
    $.jqplot.preInitHooks.push($.jqplot.AxisRangeSelector.init);
    $.jqplot.postDrawHooks.push($.jqplot.AxisRangeSelector.postDraw);
    
    function handleMouseLeave(ev, gridpos, datapos, neighbor, plot) {
        var ars = plot.plugins.axisRangeSelector;
        ars.onGrid = false;
        if (ars.show) {
            $(ev.target).css('cursor', ars.previousCursor);
            ars._selection.gridpos = gridpos;
            ars._selection.datapos = datapos;
        }
    }
    
    function handleMouseEnter(ev, gridpos, datapos, neighbor, plot) {
        var ars = plot.plugins.axisRangeSelector;
        ars.onGrid = true;
        if (ars.show) {
            ars.previousCursor = ev.target.style.cursor;
            ev.target.style.cursor = ars.style;
        }
    } 
    
    function getEventPosition(ev) {
        var plot = ev.data.plot;
        var go = plot.eventCanvas._elem.offset();
        var gridPos = {x:ev.pageX - go.left, y:ev.pageY - go.top};
        var dataPos = {xaxis:null, yaxis:null, x2axis:null, y2axis:null, y3axis:null, y4axis:null, y5axis:null, y6axis:null, y7axis:null, y8axis:null, y9axis:null, yMidAxis:null};
        var an = ['xaxis', 'yaxis', 'x2axis', 'y2axis', 'y3axis', 'y4axis', 'y5axis', 'y6axis', 'y7axis', 'y8axis', 'y9axis', 'yMidAxis'];
        var ax = plot.axes;
        var n, axis;
        for (n = 11; n > 0; n--) {
            axis = an[n-1];
            if (ax[axis].show) {
                dataPos[axis] = ax[axis].series_p2u(gridPos[axis.charAt(0)]);
            }
        }
        return {offsets:go, gridPos:gridPos, dataPos:dataPos};
    }
    
    function handleSelectionMove(ev) {
        var plot = ev.data.plot,
            ars = plot.plugins.axisRangeSelector,
            ctx, positions, gridpos, xpos, ypos, height, width, sel;
        // don't do anything if not on grid.
        if (ars.show && ars._selection.started) {
            ev.preventDefault();
            ctx = ars.selectionCanvas._ctx;
            positions = getEventPosition(ev);
            gridpos = positions.gridPos;
            datapos = positions.dataPos;
            ars._selection.gridpos = gridpos;
            ars._selection.datapos = datapos;
            ars._selection.dragging = true;
            xpos = gridpos.x;
            ypos = gridpos.y;
            height = ctx.canvas.height;
            width = ctx.canvas.width;
            if (ars.constrainTo == 'x') {
                ars._selection.end = [xpos, height];
            } else if (ars.constrainTo == 'y') {
                ars._selection.end = [width, ypos];
            } else {
                ars._selection.end = [xpos, ypos];
            }
            sel = window.getSelection;
            if (document.selection && document.selection.empty)
            {
                document.selection.empty();
            } else if (sel && !sel().isCollapsed) {
                sel().collapse();
            }
            drawSelectionBox.call(ars);
            ctx = null;
        }
    }
    
    function handleMouseDown(ev, gridpos, datapos, neighbor, plot) {
        var ars = plot.plugins.axisRangeSelector,
            axis = "xaxis";        
        if (plot.plugins.mobile) {
            $(document).one('vmouseup.jqplot_ars', {plot:plot}, handleMouseUp);
        } else {
            $(document).one('mouseup.jqplot_ars', {plot:plot}, handleMouseUp);
        }
        if (document.onselectstart !== undefined && 
                document.onselectstart !== null) {
            ars._oldHandlers.onselectstart = document.onselectstart;
            document.onselectstart = function () { 
                return false; 
            };
        }
        if (document.ondrag !== undefined && document.ondrag !== null) {
            ars._oldHandlers.ondrag = document.ondrag;
            document.ondrag = function () { 
                return false; 
            };
        }
        if (document.onmousedown !== undefined && document.onmousedown !== null) {
            ars._oldHandlers.onmousedown = document.onmousedown;
            document.onmousedown = function () { 
                return false; 
            };
        }
        if (ars.constrainTo == 'x') {
            ars._selection.start = [gridpos.x, 0];
        } else if (ars.constrainTo == 'y') {
            ars._selection.start = [0, gridpos.y];
        } else {
            ars._selection.start = [gridpos.x, gridpos.y];
        }
        ars._selection.started = true;
        for (axis in datapos) {
            // get selection starting position.
            ars._selection.axes.start[axis] = datapos[axis];
        }  
        if (plot.plugins.mobile) {
            $(document).on('vmousemove.jqplot_ars', {plot:plot}, handleSelectionMove);              
        } else {
            $(document).on('mousemove.jqplot_ars', {plot:plot}, handleSelectionMove);              
        }
    }
    
    function handleMouseUp(ev) {
        var plot = ev.data.plot,
            ars = plot.plugins.axisRangeSelector,
            xpos, ypos, datapos, height, width, axes, axis = "xaxis",
            rangeStart = 0, rangeEnd = 0, temp = 0, evt = null;
        if (ars._selection.dragging) {
            xpos = ars._selection.gridpos.x;
            ypos = ars._selection.gridpos.y;
            datapos = ars._selection.datapos;
            height = ars.selectionCanvas._ctx.canvas.height;
            width = ars.selectionCanvas._ctx.canvas.width;
            axes = plot.axes;
            
            if (ars.constrainOutsideSelection && !ars.onGrid) {
                if (xpos < 0) { 
                    xpos = 0; 
                } else if (xpos > width) { 
                    xpos = width;
                }
                if (ypos < 0) { 
                    ypos = 0; 
                } else if (ypos > height) { 
                    ypos = height; 
                }
                for (axis in datapos) {
                    if (datapos[axis]) {
                        if (axis.charAt(0) == 'x') {
                            datapos[axis] = axes[axis].series_p2u(xpos);
                        } else {
                            datapos[axis] = axes[axis].series_p2u(ypos);
                        }
                    }
                }
            }
            
            if (ars.constrainTo == 'x') {
                ypos = height;
                rangeStart = Math.floor(ars._selection.axes.start.xaxis);
                rangeEnd = Math.floor(datapos.xaxis);
            } else if (ars.constrainTo == 'y') {
                xpos = width;
                rangeStart = Math.floor(ars._selection.axes.start.yaxis);
                rangeEnd = Math.floor(datapos.yaxis);
            }
            ars._selection.end = [xpos, ypos];
            ars._selection.gridpos = {x:xpos, y:ypos};
            if (rangeStart > rangeEnd) {
                temp = rangeStart;
                rangeStart = rangeEnd;
                rangeEnd = temp;
            }
            //console.log("selected region start="+rangeStart+" end="+rangeEnd);
            evt = jQuery.Event('jqplotRangeSelected');
            evt.which = ev.which;
            evt.pageX = ev.pageX;
            evt.pageY = ev.pageY;
            plot.target.trigger(evt, [rangeStart, rangeEnd]);
        }
        ars._selection.started = false;
        ars._selection.dragging = false;
        
        $(document).off('mousemove.jqplot_ars', handleSelectionMove);
        
        if (document.onselectstart !== undefined && 
                document.onselectstart !== null &&
                ars._oldHandlers.onselectstart !== null) {
            document.onselectstart = ars._oldHandlers.onselectstart;
            ars._oldHandlers.onselectstart = null;
        }
        if (document.ondrag !== undefined && 
                document.ondrag !== null &&
                ars._oldHandlers.ondrag !== null) {
            document.ondrag = ars._oldHandlers.ondrag;
            ars._oldHandlers.ondrag = null;
        }
        if (document.onmousedown !== undefined && 
                document.onmousedown !== null &&
                ars._oldHandlers.onmousedown !== null) {
            document.onmousedown = ars._oldHandlers.onmousedown;
            ars._oldHandlers.onmousedown = null;
        }
        if (evt) {
            plot.target.trigger(evt, [rangeStart, rangeEnd]);
        }
    }
    
    function drawSelectionBox() {
        var start = this._selection.start,
            end = this._selection.end,
            ctx = this.selectionCanvas._ctx,
            l, t, h, w;
        if (end[0] > start[0]) {
            l = start[0];
            w = end[0] - start[0];
        } else {
            l = end[0];
            w = start[0] - end[0];
        }
        if (end[1] > start[1]) {
            t = start[1];
            h = end[1] - start[1];
        } else {
            t = end[1];
            h = start[1] - end[1];
        }
        ctx.fillStyle = 'rgba(0,0,0,0.2)';
        ctx.strokeStyle = '#999999';
        ctx.lineWidth = 1.0;
        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);
        ctx.fillRect(0, 0, ctx.canvas.width, ctx.canvas.height);
        ctx.clearRect(l, t, w, h);
        // IE won't show transparent fill rect, so stroke a rect also.
        ctx.strokeRect(l, t, w, h);
        ctx = null;
    }

})(jQuery);
