//Copyright (c) InMon Corp. ALL RIGHTS RESERVED

(function($) {
    /**
     * class: $.jqplot.GridLegendRenderer
     * A legend renderer rather like EnhancedLegendRenderer that allows
     * configuration of number of rows and columns of legend items, with legend
     * items being shown with different styling if the associated series is
     * not currently shown in the plot. However this legend renderer
     * does not support toggleSeries, instead it allows a clickHandler to
     * be configured. This clickHandler can be used to remove a series from
     * the plot (ie set series.show = false) and then replot the plot, 
     * rescaling the axes. EnhancedLegendRenderer uses series.toggleSeries
     * which just hides the series plot which still takes up space, so it does
     * not work for stacked series.
     * Example usage:
     * plot2 = $.jqplot("chart2", [s1, s2], {
     *      legend: {
     *           renderer: $.jqplot.GridLegendRenderer,
     *           rendererOptions: {
     *               numberRows: plotData.series.length === 2 ? 1 : 2,
     *               clickHandler: handleLegendClick
     *           },
     *           show: true,
     *           labels: plotData.legendLabels,
     *           placement: "outsideGrid",
     *           location: "s"    
     *       }
     * });
     * function handleLegendClick(evt) {
     *       var data = evt.data,
     *           series = data.series,
     *           plot = data.plot,
     *           sidx = series.index;
     *       plot.options.series[sidx].show = !series.show;
     *       opts = {resetAxes: true, recompute: true};
     *       plot.replot(opts);
     *   }
     */    

    $.jqplot.GridLegendRenderer = function(){
        $.jqplot.TableLegendRenderer.call(this);
    };
    
    $.jqplot.GridLegendRenderer.prototype = 
        new $.jqplot.TableLegendRenderer();
    $.jqplot.GridLegendRenderer.prototype.constructor = 
        new $.jqplot.GridLegendRenderer();
    
 // called with scope of legend.
    $.jqplot.GridLegendRenderer.prototype.init = function(options) {
        // prop: numberRows
        // Maximum number of rows in the legend.  0 or null for unlimited.
        this.numberRows = null;
        // prop: numberColumns
        // Maximum number of columns in the legend.  0 or null for unlimited.
        this.numberColumns = null;
        // prop: clickHandler
        // Function called when legend item is clicked. Click is not enabled
        // if handler is not given. The clickHandler is added as an event
        // handler to the dom legend items and the event data object include
        // properties:
        // series - the series associated with the clicked legend item
        // plot - jqplot object
        this.clickHandler = null;
        $.extend(true, this, options);
        
        if (this.clickHandler) {
            $.jqplot.postDrawHooks.push(postDraw);
        }
    };
    
    // called with scope of legend
    $.jqplot.GridLegendRenderer.prototype.draw = function(offsets, plot) {
        if (this.show) {
            var series = this._series;
            var s;
            var ss = 'position:absolute;';
            ss += (this.background) ? 'background:'+this.background+';' : '';
            ss += (this.border) ? 'border:'+this.border+';' : '';
            ss += (this.fontSize) ? 'font-size:'+this.fontSize+';' : '';
            ss += (this.fontFamily) ? 'font-family:'+this.fontFamily+';' : '';
            ss += (this.textColor) ? 'color:'+this.textColor+';' : '';
            ss += (this.marginTop !== undefined && this.marginTop !== null) ? 
                    'margin-top:'+this.marginTop+';' : '';
            ss += (this.marginBotton !== undefined && this.marginBottom !== null) ? 
                    'margin-bottom:'+this.marginBottom+';' : '';
            ss += (this.marginLeft !== undefined && this.marginLeft !== null) ? 
                    'margin-left:'+this.marginLeft+';' : '';
            ss += (this.marginRight !== undefined && this.marginRight !== null) ? 
                    'margin-right:'+this.marginRight+';' : '';
            this._elem = $('<table class="jqplot-table-legend" style="'+ss+'"></table>');
            if (this.clickHandler) {
                this._elem.css('z-index', '3');
            }
        
            var pad = false, 
                reverse = false,
                nr, nc,
                i, j, tr, td1, td2, lt, rs, div0, div1,
                idx = 0;
            if (this.numberRows) {
                nr = this.numberRows;
                if (!this.numberColumns) {
                    nc = Math.ceil(series.length/nr);
                } else {
                    nc = this.numberColumns;
                }
            } else if (this.numberColumns) {
                nc = this.numberColumns;
                nr = Math.ceil(series.length/this.numberColumns);
            } else {
                nr = series.length;
                nc = 1;
            }
                
            // check to see if we need to reverse
            for (i = series.length-1; i >= 0; i--) {
                if (nc == 1 && series[i]._stack || 
                        series[i].renderer.constructor == $.jqplot.BezierCurveRenderer) {
                    reverse = true;
                }
            }    
                
            for (i = 0; i < nr; i++) {
                tr = $(document.createElement('tr'));
                tr.addClass('jqplot-table-legend');
                if (reverse) {
                    tr.prependTo(this._elem);
                } else {
                    tr.appendTo(this._elem);
                }
                for (j = 0; j < nc; j++) {
                    if (idx < series.length && 
                            (series[idx].show || series[idx].showLabel)) {
                        s = series[idx];
                        lt = this.labels[idx] || s.label.toString();
                        if (lt) {
                            var color = s.color;
                            if (!reverse) {
                                if (i > 0) {
                                    pad = true;
                                } else {
                                    pad = false;
                                }
                            } else {
                                if (i == nr-1) {
                                    pad = false;
                                } else {
                                    pad = true;
                                }
                            }
                            rs = (pad) ? this.rowSpacing : '0';
                            
                            td1 = $(document.createElement('td'));
                            td1.addClass('jqplot-table-legend jqplot-table-legend-swatch');
                            td1.css({textAlign: 'center', paddingTop: rs});

                            div0 = $(document.createElement('div'));
                            div0.addClass('jqplot-table-legend-swatch-outline');
                            div1 = $(document.createElement('div'));
                            div1.addClass('jqplot-table-legend-swatch');
                            div1.css({backgroundColor: color, borderColor: color});

                            td1.append(div0.append(div1));

                            td2 = $(document.createElement('td'));
                            td2.addClass('jqplot-table-legend jqplot-table-legend-label');
                            td2.css('paddingTop', rs);
                    
                            if (this.escapeHtml) {
                                td2.text(lt);
                            } else {
                                td2.html(lt);
                            }
                            if (reverse) {
                                if (this.showLabels) {
                                    td2.prependTo(tr);
                                }
                                if (this.showSwatches) {
                                    td1.prependTo(tr);
                                }
                            } else {
                                if (this.showSwatches) {
                                    td1.appendTo(tr);
                                }
                                if (this.showLabels) {
                                    td2.appendTo(tr);
                                }
                            }
                            
                            if (this.clickHandler) {
                                if (this.showSwatches) {
                                    td1.on("click", {series: s, plot: plot}, 
                                            this.clickHandler);
                                    td1.addClass('jqplot-seriesToggle');
                                }
                                if (this.showLabels)  {
                                    td2.on("click", {series: s, plot: plot}, 
                                            this.clickHandler);
                                    td2.addClass('jqplot-seriesToggle');
                                }

                                // for series that are already hidden, add the hidden class
                                if (!s.show && s.showLabel) {
                                    td1.addClass('jqplot-series-hidden');
                                    td2.addClass('jqplot-series-hidden');
                                }
                            }
                            pad = true;
                        }
                    }
                    idx++;
                }
                td1 = td2 = div0 = div1 = null;   
            }
        }
        return this._elem;
    };
    
    // called with scope of plot.
    postDraw = function () {
        if (this.legend.renderer.constructor == $.jqplot.GridLegendRenderer && 
                this.legend.clickHandler) {
            var e = this.legend._elem.detach();
            this.eventCanvas._elem.after(e);
        }
    };
  
    
})(jQuery);