//Copyright (c) InMon Corp. ALL RIGHTS RESERVED


(function($) {
    /**
     * class: $.jqplot.MirrorSeriesRenderer
     * Renderers mirrored series (ie paired positive and negative series) as
     * a series of bars with the negative bars filled with a darker shade
     * Mirrored series must be specified with the index of the series to mirror
     * with. 
     * jqplot must be initialised with the series ordered with all positive
     * series first, followed by all corresponding mirrored series. Values for
     * mirrored series must be negative (or zero).
     * Can be used to render a stepped line chart (ie line graph with a step 
     * between adjacent points rather than a line joining adjacent points),
     * by setting options.showAsLine = true.
     * Assumes xaxis is horizontal and is the primary axis, while the yaxis is
     * the stack axis.
     * Set plot.options.showMirrorLabels = true if the legend is to show entries
     * for the negative series, leave as undefined or false to only show
     * positive series labels in the legend.
     * The series show property is also use to decide whether to actually draw
     * a series in the plot and include its values for stacking. Standard
     * behaviour uses the show property to control whether the series canvas is
     * showing and includes a hidden series' values in the stack.
     * 
     * Example usage:
     * plot2 = $.jqplot("chart2", [s1, s2], {
     *      stackSeries: true,
     *      showMirrorLabels: true,
     *      seriesDefaults: {
     *           renderer:$.jqplot.MirrorSeriesRenderer,
     *           rendererOptions: {
     *             showAsLine: true, //filled stacked area chart (since fill == true),
     *             //showAsLine: false //filled stacked bar chart
     *             highlightMouseOver: false
     *           },
     *           fill: true,
     *           shadow: true,
     *           fillAndStroke: false,
     *           showMarker: false,
     *           fillToZero: true
     *       },
     *       seriesColors: ["blue","cyan", "yellow", "magenta", "red"],
     *       series: [{label: "series1"}, {label: "series2", mirrorIndex: 0}] //s2 is the mirror series for s1
     * });
     * 
     */
    
    $.jqplot.MirrorSeriesRenderer = function() {
        $.jqplot.LineRenderer.call(this);
    };
    
    $.jqplot.MirrorSeriesRenderer.prototype = new $.jqplot.LineRenderer();
    $.jqplot.MirrorSeriesRenderer.prototype.constructor = 
        new $.jqplot.MirrorSeriesRenderer();
    
    //called with scope of series
    $.jqplot.MirrorSeriesRenderer.prototype.init = function(options, plot) {
        options = options || {};
        this._type = 'mirrorSeries';
        //prop: mirrorIndex
        //index of the series that this is a mirror of (ie the positive series
        //that this negative series corresponds with.
        //this.mirrorIndex;
        //prop: showAsLine
        //if true, renders the series as a stepped line chart and barPadding
        //is ignored and set to 0. Otherwise renders as a series of bars.
        this.showAsLine = false;
        //prop: other
        //true if this is the special other series that is to be rendered using
        //color this._otherColor
        //this.other
        // prop: barPadding
        // Number of pixels between adjacent bars at the same axis value.
        this.barPadding = 2;
        this.barWidth = null;
        // prop: fill
        // True to fill the series (either bars or stacked line).
        //this.fill = true;
        // prop: highlightMouseOver
        // True to highlight bar when moused over.
        // Highlighting not supported when showAsLine = true 
        this.highlightMouseOver = true;
        // prop: highlightMouseDown
        // True to highlight when a mouse button is pressed over a slice.
        // This will be disabled if highlightMouseOver is true.
        this.highlightMouseDown = false;
        // prop: highlightColors
        // an array of colors to use when highlighting a slice.
        this.highlightColors = [];
        // if user has passed in highlightMouseDown option and not set highlightMouseOver, disable highlightMouseOver
        if (options.highlightMouseDown && options.highlightMouseOver === null) {
            options.highlightMouseOver = false;
        }
        
        $.extend(true, this, options);
        if (this.mirrorIndex !== undefined && !plot.options.showMirrorLabels) {
            this.showLabel = false;
        }
        this._otherColor = "#ccc";
        if (this.other) {
            this.color = this._otherColor;
        } else {
            positiveColors = new $.jqplot.ColorGenerator(this.seriesColors);
            this.color = positiveColors.get(this.index);
            if (this.mirrorIndex !== undefined) {
                this.color = colorLuminance(positiveColors.get(this.mirrorIndex), -0.2);
            }
        }
        if (this.showAsLine) {
            this.barPadding = 0;
        } 
        
        // fill is still needed to properly draw the legend.
        // bars have to be filled.
        //this.fill = true;
        //only support vertical bars
        this._primaryAxis = '_xaxis';
        this._stackAxis = 'y';
        this.fillAxis = 'y';
        // index of the currently highlighted point, if any
        this._highlightedPoint = null;
        // Array of actual data colors used for each data point.
        this._dataColors = [];
        //array (or array of array) of points used to draw on canvas
        this._gridPoints = [];
        
        // set the shape renderer options
        var opts = {lineJoin: 'miter', lineCap: 'round', fill: this.fill, 
                isarc: false, strokeStyle: this.color, fillStyle: this.color, 
                closePath: this.fill};
        this.renderer.shapeRenderer.init(opts);
        // set the shadow renderer options
        var sopts = {lineJoin: 'miter', lineCap: 'round', fill: this.fill, 
                isarc: false, angle: this.shadowAngle, 
                offset: this.shadowOffset, alpha: this.shadowAlpha, 
                depth: this.shadowDepth, closePath: this.fill};
        this.renderer.shadowRenderer.init(sopts);
        
        plot.postSeriesInitHooks.addOnce(postSeriesInit);
        plot.postDrawHooks.addOnce(postPlotDraw);
        plot.eventListenerHooks.addOnce('jqplotMouseMove', handleMove);
        plot.eventListenerHooks.addOnce('jqplotClick', handleClick);
    };
    
    //have to do this globally and not per plot instance in renderer init,
    //because parseOptions is called before renderer is initialised.
    $.jqplot.postParseOptionsHooks.push(postParseOptions);
    
    /**
     * Called with scope of plot.
     * Called after series have been created with their options, but
     * series.init not yet called so renderer is not initialised. However
     * mirrorIndex for series will have been set, so we can determine the
     * min mirrorIndex for the series (includes !show since !show series
     * are drawn with zero height).
     * Also called before computePlotData, so set disableStack for the series
     * with the min mirrorIndex (if any) so that can be taken into account
     * properly in computePlotData.
     */
    function postParseOptions(options) {
        var i, s,
            minMirrorIndex = 0, 
            seriesIndex = 0, 
            mirroring = false;
        for (i = 0; i < this.series.length; i++) {
            s = this.series[i];
            if (s.mirrorIndex !== undefined && s.mirrorIndex !== null) {
                mirroring = true;
                if (s.mirrorIndex <= minMirrorIndex) {
                    minMirrorIndex = s.mirrorIndex;
                    seriesIndex = i;
                }
            }
        }
        if (mirroring) {
            //console.log("set min mirrorIndex="+minIndex);
            this.minMirrorIndex = minMirrorIndex;
            this.series[seriesIndex].disableStack = true;
        }
    }
    
    
    $.jqplot.MirrorSeriesRenderer.prototype.setBarWidth = function() {
        //need to know how many data values we have on the axis 
        //and work out bar width
        var nvals = this.data.length;
        var paxis = this[this._primaryAxis];
        var temp = (this.barPadding === 0 && !this.showAsLine) ? 1.0 : 0;
        this.barWidth = (paxis._offsets.max - paxis._offsets.min) / nvals - this.barPadding + temp;
    };
    
    /**
     * Lighten or darken a color by changing its luminosity
     * @param color to darken
     * @param lum the luminosity factor,eg -0.1 is 10% darker, 
     * 0.2 is 20% lighter
     */
    function colorLuminance(color, lum) {
        lum = lum || 0;
        // convert to decimal and change luminosity
        var rgba = $.jqplot.getColorComponents(color),
            newrgb = '#',
            i, c;
        for (i = 0; i < 3; i++) {
            c = rgba[i];
            c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
            newrgb += ("00"+c).substr(c.length);
        }
        return newrgb;
    }

    function getStart(sidx, didx, comp, plot) {
        // check if sign change
        var seriesIndex = sidx,
            prevSeriesIndex = sidx - 1,
            prevVal,
            aidx = 1; //y axis

        // is this not the first series?
        if (seriesIndex > 0) {
            prevVal = plot.series[prevSeriesIndex]._plotData[didx][aidx];
            // is there a sign change
            if ((comp * prevVal) < 0) {
                start = getStart(prevSeriesIndex, didx, comp, plot);
            } else { //no sign change
                start = plot.series[prevSeriesIndex].gridData[didx][aidx];
            }
        } else { //if first series, return value at 0
            start = (aidx === 0) ? 
                    plot.series[seriesIndex]._xaxis.series_u2p(0) : 
                        plot.series[seriesIndex]._yaxis.series_u2p(0);
        }
        return start;
    }
    
    $.jqplot.MirrorSeriesRenderer.prototype.draw = function(ctx, gridData, options, plot) {
        if (this.showAsLine) {
            this.renderer.drawLine.call(this, ctx, gridData, options, plot);
        } else {
            this.renderer.drawBars.call(this, ctx, gridData, options, plot);
        }
    };
    
    $.jqplot.MirrorSeriesRenderer.prototype.drawBars = function(ctx, gridData, options, plot) {
        var i;
        //have to make a copy of options because it may be modified later.
        var opts = $.extend({}, options);
        var shadow = (opts.shadow !== undefined && opts.shadow !== null) ? 
                opts.shadow : this.shadow;
        var showLine = (opts.showLine !== undefined && opts.showLine !== null) ? 
                opts.showLine : this.showLine;
        // clear out data colors.
        this._dataColors = [];
        this._gridPoints = [];
        var points = [], base, ystart, clr, temp;
        
        if (this.barWidth === undefined || this.barWidth === null) {
            this.renderer.setBarWidth.call(this);
        }
        this._barNudge = 0;

        if (showLine) {
            for (i = 0; i < gridData.length; i++) {
                if (!this._stack && this.data[i][1] === null) {
                    continue;
                }
                points = [];
                base = gridData[i][0] + this._barNudge;

                // stacked
                if (this._stack && this._prevGridData.length) {
                    ystart = getStart(this.index, i, this._plotData[i][1], plot);
                } else { //not stacked
                    if (this.fillToZero) {
                        ystart = this._yaxis.series_u2p(0);
                    } else {
                        ystart = ctx.canvas.height;
                    }
                }

                if (!this.fillToZero || this._plotData[i][1] >= 0) { 
                    points.push([base-this.barWidth/2, ystart]);
                    points.push([base-this.barWidth/2, gridData[i][1]]);
                    points.push([base+this.barWidth/2, gridData[i][1]]);
                    points.push([base+this.barWidth/2, ystart]);
                } else {
                    // for negative bars make sure points are always ordered clockwise              
                    points.push([base-this.barWidth/2, gridData[i][1]]);
                    points.push([base-this.barWidth/2, ystart]);
                    points.push([base+this.barWidth/2, ystart]);
                    points.push([base+this.barWidth/2, gridData[i][1]]);
                }
                this._gridPoints.push(points);
                // now draw the shadows if not stacked.
                // for stacked plots, they are predrawn by drawShadow
                if (shadow && !this._stack) {
                    var sopts = $.extend(true, {}, opts);
                    delete sopts.fillStyle;
                    this.renderer.shadowRenderer.draw(ctx, points, sopts);
                }
                clr = opts.fillStyle || this.color;
                this._dataColors.push(clr);
                this.renderer.shapeRenderer.draw(ctx, points, opts); 
            }
            if (this.highlightColors.length === 0) {
                this.highlightColors = $.jqplot.computeHighlightColors(this._dataColors);
            }           
            else if (typeof(this.highlightColors) == 'string') {
                temp = this.highlightColors;
                this.highlightColors = [];
                for (i = 0; i < this._dataColors.length; i++) {
                    this.highlightColors.push(temp);
                }
            }
        }
    };
    
    /**
     * Called with scope of series.
     * Sets the points that are actually used to draw the series when
     * showAsLine == true
     */
    $.jqplot.MirrorSeriesRenderer.prototype.setGridPoints = function(plot, gridData) {
        var gdLength = null, data = [],
            i, x, y;
        if (!gridData) {
            if (!this._stack) {
                data = this.data;
            } else {
                data = this._plotData;
            }
            gridData = this.renderer.makeGridData.call(this, data, plot);
        }
        gdLength = gridData.length;
        if (this.barWidth === undefined || this.barWidth === null) {
            this.renderer.setBarWidth.call(this);
        }
        if (this.showLine && gdLength) {
            this._gridPoints = [];
            x = gridData[0][0] - this.barWidth/2;
            y = gridData[0][1];
            this._gridPoints.push([x, y]);
            x += this.barWidth;
            this._gridPoints.push([x, y]);
            for (i = 1; i < gdLength; i++) {
                y = gridData[i][1];
                this._gridPoints.push([x, y]);
                x += this.barWidth;
                this._gridPoints.push([x, y]);
            }
        }
    };
        
    $.jqplot.MirrorSeriesRenderer.prototype.drawLine = function(ctx, gridData, options, plot) {
        var i;
        //have to make a copy of options because it may be modified later.
        var opts = $.extend({}, options);
        var shadow = (opts.shadow !== undefined && opts.shadow !== null) ? 
                opts.shadow : this.shadow;
        var showLine = (opts.showLine !== undefined && opts.showLine !== null) ? 
                opts.showLine : this.showLine;
        var fill = (opts.fill !== undefined && opts.fill !== null) ? 
                opts.fill : this.fill;
        var fillAndStroke = (opts.fillAndStroke !== undefined && 
                opts.fillAndStroke !== null) ? 
                opts.fillAndStroke : this.fillAndStroke;
        var stack = this._stack && this._prevGridData.length;
        var points = [];
        var gdLength = gridData.length;
        var pyzero = 
            this.fillToZero ? this._yaxis.series_u2p(0) : ctx.canvas.height;
        var prevSeries, prevGridPoints, fasopts;
        
        if (showLine && gdLength) {
            this.renderer.setGridPoints.call(this, plot, gridData);
            points = this._gridPoints.slice(0); //copy _gridPoints
            if (fill) {
                if (!stack) {
                    //return to yzero
                    points.push([points[points.length-1][0], pyzero]);
                    points.push([points[0][0], pyzero]);
                } else { 
                    //filled and stacked so fill to line below so close
                    //shape with previous area points
                    prevSeries = plot.series[this.index-1];
                    if (!prevSeries.show) {
                        //draw is only called for showing series, so if !show
                        //_gridPoints will not have been recalculated, so
                        //do it now
                        this.renderer.setGridPoints.call(prevSeries, plot);
                    }
                    prevGridPoints = prevSeries._gridPoints;
                    for (i = prevGridPoints.length; i > 0; i--) {
                        points.push(prevGridPoints[i-1]);
                    }
                }
            }
            //shadows for stacked series are predrawn
            if (shadow && !stack) {
                this.renderer.shadowRenderer.draw(ctx, points, opts);
            }
            this.renderer.shapeRenderer.draw(ctx, points, opts);
            if (fillAndStroke) {
                fasopts = $.extend(true, {}, opts, {fill:false, closePath:false});
                this.renderer.shapeRenderer.draw(ctx, this._gridPoints, fasopts);
            }
        }
    };
    
    //for stacked plots, shadows will be pre-drawn by drawShadow
    $.jqplot.MirrorSeriesRenderer.prototype.drawShadow = function(ctx, gridData, options, plot) {
        if (this.showAsLine) {
            this.renderer.drawLineShadow.call(this, ctx, gridData, options, plot);
        } else {
            this.renderer.drawBarsShadow.call(this, ctx, gridData, options, plot);
        }
    };
    
    $.jqplot.MirrorSeriesRenderer.prototype.drawBarsShadow = function(ctx, gridData, options, plot) {    
        var opts = (options !== undefined && options !== null) ? options : {};
        var shadow = (opts.shadow !== undefined && opts.shadow !== null) ? 
                opts.shadow : this.shadow;
        var showLine = (opts.showLine !== undefined && opts.showLine !== null) ? 
                opts.showLine : this.showLine;
        var pyzero = this._yaxis.series_u2p(0);
        var base, ystart, points, i;
        
        if (this._stack && shadow) {
            if (this.barWidth === undefined || this.barWidth === null) {
                this.renderer.setBarWidth.call(this);
            }
        
            this._barNudge = 0;
            if (showLine) {
                for (i = 0; i < gridData.length; i++) {
                    if (this.data[i][1] === null) {
                        continue;
                    }
                    points = [];
                    base = gridData[i][0] + this._barNudge;
                    if (this._stack && this._prevGridData.length) {
                        ystart = getStart(this.index, i, this._plotData[i][1], plot);
                    } else {
                        if (this.fillToZero) {
                            ystart = pyzero;
                        } else {
                            ystart = ctx.canvas.height;
                        }
                    }
                    points.push([base-this.barWidth/2, ystart]);
                    points.push([base-this.barWidth/2, gridData[i][1]]);
                    points.push([base+this.barWidth/2, gridData[i][1]]);
                    points.push([base+this.barWidth/2, ystart]);
                    this.renderer.shadowRenderer.draw(ctx, points, opts);
                }
            }   
        }
    };
    
    $.jqplot.MirrorSeriesRenderer.prototype.drawLineShadow = function(ctx, gridData, options, plot) {
        var opts = (options !== undefined && options !== null) ? options : {};
        var shadow = (opts.shadow !== undefined && opts.shadow !== null) ? 
                opts.shadow : this.shadow;
        var showLine = (opts.showLine !== undefined && opts.showLine !== null) ?
                opts.showLine : this.showLine;
        var fill = (opts.fill !== undefined && opts.fill !== null) ? 
                opts.fill : this.fill;
        var stack = this._stack && this._prevGridData.length;
        var points = [], i;
        var gdLength = gridData.length;
        var prevSeries, prevGridPoints;
        
        if (stack && fill && shadow) {
            if (showLine && gdLength) {
                this.renderer.setGridPoints.call(this, plot, gridData);
                points = this._gridPoints.slice(0); //copy contents of ._gridPoints
                //filled and stacked so fill to line below so close
                //shape with previous area points
                prevSeries = plot.series[this.index-1];
                if (!prevSeries.show) {
                    //make sure _gridPoints has been set. They may not
                    //have been if the previous series is not showing
                    this.renderer.setGridPoints.call(prevSeries, plot);
                }
                prevGridPoints = prevSeries._gridPoints;
                for (i = prevGridPoints.length; i > 0; i--) {
                    points.push(prevGridPoints[i-1]);
                }
                this.renderer.shadowRenderer.draw(ctx, points, opts);
            }  
        }
    };
    
    /**
     * Called with scope of series.
     * Recalculates plotData to take into account whether the series is
     * shown. Standard behaviour of show controls the hidden state of the 
     * series canvas, we are also using it to control whether the series is 
     * drawn in the plot and its values included for stacking.
     * Since we don't know whether a previous series show state has changed,
     * we always recompute plotData if the series is stacked.
     * Assumes plotData has already been set via plot.computePlotData (which
     * does not take into account show property).
     */
    function postSeriesInit(target, data, seriesDefaults, options) {
        var sidx = this._stackAxis === 'x' ? 0 : 1,
            pdLength = this._plotData.length,
            stack = this._stack && this._prevPlotData.length,
            i;
        if (!this.show) {
            if (stack) {
                for (i = 0; i < pdLength; i++) {
                    this._plotData[i][sidx] = this._prevPlotData[i][sidx];
                    this._stackData[i][sidx] = this._plotData[i][sidx];
                }
            } else {
                //if this series is not stacked, we need to zero out the
                //plotData in case there is a series stacked on top and also
                //to make sure the axis scale is set correctly.
                for (i = 0; i < pdLength; i++) {
                    this._plotData[i][sidx] = 0;
                    this._stackData[i][sidx] = this._plotData[i][sidx];
                    if (this._primaryAxis == "_xaxis") {
                        this._plotValues.y[i] = 0;
                    } else {
                        this._plotValues.x[i] = 0;
                    }
                }
            }
        } else {
            if (stack) {
                //prevPlotData may have changed if the previous series is now hidden 
                for (i = 0; i < pdLength; i++) {
                    this._plotData[i][sidx] = this.data[i][sidx] + this._prevPlotData[i][sidx];
                    this._stackData[i][sidx] = this._plotData[i][sidx];
                }
            }
        } 
    }
    
    /**
     * Called with context of plot
     */
    function postPlotDraw() {
        // Memory Leaks patch    
        if (this.plugins.mirrorSeriesRenderer && 
                this.plugins.mirrorSeriesRenderer.highlightCanvas) {
            this.plugins.mirrorSeriesRenderer.highlightCanvas.resetCanvas();
            this.plugins.mirrorSeriesRenderer.highlightCanvas = null;
        }
         
        this.plugins.mirrorSeriesRenderer = {highlightedSeriesIndex:null};
        this.plugins.mirrorSeriesRenderer.highlightCanvas = new $.jqplot.GenericCanvas();
        
        this.eventCanvas._elem.before(this.plugins.mirrorSeriesRenderer.highlightCanvas.
                createElement(this._gridPadding, 
                        'jqplot-mirrorSeriesRenderer-highlight-canvas', 
                        this._plotDimensions, this));
        this.plugins.mirrorSeriesRenderer.highlightCanvas.setContext();
        //this.eventCanvas._elem.on('mouseleave', {plot: this}, function (ev) { 
        //        unhighlight(ev.data.plot);  
        //   });
    } 
    
    /**
     * Checks whether gridPos is within any bar, and if so returns 
     * an object with properties seriesIndex, pointIndex, where pointIndex
     * is the barIndex (category).
     */
    function getBar(gridPos, plot) {
        var series = plot.series,
            x = gridPos.x,
            y = gridPos.y,
            stackLength = plot.seriesStack.length,
            points, s, i, j, k;
        i = plot.seriesStack[stackLength-1];
        s = series[i];
        for (j = 0; j < s._gridPoints.length; j++) {
            points = s._gridPoints[j];
            if (x > points[0][0] && x < points[2][0]) {
                //we've found the barIndex (category)
                //now check for containment within the stacked bars
                for (k = 0; k < stackLength; k++) {
                    i = plot.seriesStack[k];
                    s = series[i];
                    points = s._gridPoints[j];
                    if (y >= points[2][1] && y <= points[0][1]) {
                        //console.log("getBarIndex bar="+j+" series="+k);
                        return {seriesIndex: k, pointIndex: j};
                    }
                }
            }
        }
        return null;
    }
    
    /**
     * Returns the index in the series of the point closest to gridPos.
     * y coordinate is ignored.
     * Returns null if not close to a plotted point (in the x dimension)
     * Used when showAsLine=true
     */
    function getPointIndex(gridPos, plot) {
        var series = plot.series,
            x = gridPos.x,
            wobble = 2,
            s, i, gridx;
        if (series.length === 0) {
            return null;
        }
        s = series[0];
        wobble = s.barWidth / 4;
        for (i = 0; i < s.gridData.length; i++) {
            gridx = s.gridData[i][0];
            if (x > gridx-wobble && x < gridx+wobble) {
                return i;
            }
        }
        return null;
    }
    
    function highlight(plot, sidx, pidx) {
        var i, s, points, opts;
        var canvas = plot.plugins.mirrorSeriesRenderer.highlightCanvas;
        canvas._ctx.clearRect(0,0,canvas._ctx.canvas.width, canvas._ctx.canvas.height);
        plot.plugins.mirrorSeriesRenderer.highlightedCategoryIndex = pidx;
        for (i = 0; i < plot.series.length; i++) {
            s = plot.series[i];
            opts = {fillStyle: s.highlightColors[pidx], fillRect: false};
            points = s._gridPoints[pidx];
            s.renderer.shapeRenderer.draw(canvas._ctx, points, opts);
        }
        canvas = null;
    }
    
    function unhighlight(plot) {
        var canvas = plot.plugins.mirrorSeriesRenderer.highlightCanvas;
        canvas._ctx.clearRect(0,0, canvas._ctx.canvas.width, canvas._ctx.canvas.height);
        plot.plugins.mirrorSeriesRenderer.highlightedCategoryIndex = null;
        plot.target.trigger('jqplotDataUnhighlight');
        canvas =  null;
    } 
    
    function handleMove(ev, gridPos, dataPos, neighbor, plot) {
        var bar = null;
        if (!plot.series[0].showAsLine) {
            bar = getBar(gridPos, plot);
        }
        if (bar !== null) {
            var ins = [bar.seriesIndex, bar.pointIndex];
            var evt1 = jQuery.Event('jqplotDataMouseOver');
            evt1.pageX = ev.pageX;
            evt1.pageY = ev.pageY;
            plot.target.trigger(evt1, ins);
            if (plot.series[ins[0]].show && plot.series[ins[0]].highlightMouseOver &&
                    ins[1] !== plot.plugins.mirrorSeriesRenderer.highlightedCategoryIndex) {
                var evt = jQuery.Event('jqplotDataHighlight');
                evt.which = ev.which;
                evt.pageX = ev.pageX;
                evt.pageY = ev.pageY;
                plot.target.trigger(evt, ins);
                highlight(plot, bar.seriesIndex, bar.pointIndex);
            }
        } else if (bar === null) {
            unhighlight(plot);
        }
    }

    function handleClick(ev, gridPos, dataPos, neighbor, plot) {
        var bar, pointIndex = null, seriesIndex = null;
        if (plot.series[0].showAsLine) {
            pointIndex = getPointIndex(gridPos, plot);
        } else {
            bar = getBar(gridPos, plot);
            if (bar !== null) {
                pointIndex = bar.pointIndex;
                seriesIndex = bar.seriesIndex;
            }
        }
        if (pointIndex !== null) {
            var ins = [seriesIndex, pointIndex];
            var evt = jQuery.Event('jqplotDataClick');
            evt.which = ev.which;
            evt.pageX = ev.pageX;
            evt.pageY = ev.pageY;
            plot.target.trigger(evt, ins);
        }
    }
    
})(jQuery);
