//Copyright (c) InMon Corp. ALL RIGHTS RESERVED

(function($) {
    /**
     * class: $.jqplot.MultiFieldLegendRenderer
     * Renders a legend as a table with a column for each field.
     * Expects and array of arrays as the label property of legend, with
     * one sub-array for each row in the legend table. If the legend fields
     * are to be clickable each sub-array must include an array for each field
     * of the form [data, label] where label is displayed in the table cell
     * and data is given as the value property in the data object of the click
     * event. If the data array element is null, click is disabled for the field.
     * Can be configured with a clickHandler which will be called when a
     * table data cell is clicked. The data object in the click event has
     * properties colIdx and value.
     * 
     * Example usage:
     * plot2 = $.jqplot("chart2", [s1, s2], {
     *      legend: {
     *          show: true,
     *          location: "n",
     *              placement: "outsideGrid",
     *              labels: legendLabels,
     *              renderer: $.jqplot.MultiFieldLegendRenderer,
     *              rendererOptions: {
     *                  legendHeadings: legendHeadings, //adds a header to the table, leave out for no header
     *                  numberSeries: settings.truncate,
     *                  clickHandler: handleLegendClick
     *              }
     *       },
     *   });
     *   function handleLegendClick(evt) {
     *       console.log("legendClicked on column: "+evt.data.colIdx+" key="+evt.data.value);
     *   }
     * 
     * legend.label format:
     * [[[datar0-f0, labelr0-f0], [datar0-f1, labelr0-f1]...], //s1 labels
     *  [[datar1-f0, labelr1-f0], [datar1-f1, labelr1-f1]...], //s2 labels
     *  ....
     *  ]
     * 
     */
    
    $.jqplot.MultiFieldLegendRenderer = function(){
        $.jqplot.TableLegendRenderer.call(this);
    };
    
    $.jqplot.MultiFieldLegendRenderer.prototype = 
        new $.jqplot.TableLegendRenderer();
    $.jqplot.MultiFieldLegendRenderer.prototype.constructor = 
        new $.jqplot.MultiFieldLegendRenderer();
    
 // called with scope of legend.
    $.jqplot.MultiFieldLegendRenderer.prototype.init = function(options) {
        // prop: numberFields
        // Maximum number of fields in each legend entry.
        this.numberFields = null;
        // prop: numberSeries
        // Minimum number of entries/rows (one per series) to show in legend.
        // If there are fewer series, blank rows will be shown.
        // If null the number of entries/rows will be the same as the number of
        // series.
        this.numberSeries = null;
        // prop: legendHeadings
        // Headings for field columns in the legend
        this.legendHeadings = [];
        // prop: clickHandler
        // Function called when legend fields are clicked. Click is not enabled
        // if handler is not given. he clickHandler is added as an event
        // handler to the dom legend items and the event data object include
        // properties:
        // colIdx - index of the column clicked in the legend table (excludes
        // swatch column, ie first labelled column has index 0).
        // value - data associated with the field and given in labels.
        this.clickHandler = null;
        $.extend(true, this, options);
        
        if (this.clickHandler) {
            $.jqplot.postDrawHooks.push(postDraw);
        }
        if (this.numberFields !== null) {
            this._numberCols = this.numberFields; //excludes swatch column
        } else {
            this._numberCols = this.legendHeadings.length;
        }
    };
    
    /**
     * Adds a data or header row to the legend table.
     * opts properties:
     * labelData: labels for the fields in the legend
     * color: color of the swatch (used if this.showSwatches)
     * pad: whether to add row spacing (this.rowSpacing)
     * reverse: true - prepend to the table, false - append to the table
     * header: add as a header row
     * Called with scope of legend
     */   
    function addRow(opts) {
        opts = opts || {};
        var labelData = opts.labelData === undefined ? [] : opts.labelData,
            ldLength = labelData.length,
            rs = (opts.pad) ? this.rowSpacing+'px' : '0px',
            elem = (opts.header) ? this._elem : this._elem.find("tbody"),
            tr,
            td,
            div0,
            div1, 
            i, label, dataVal;
        if (!ldLength && this._numberCols !== undefined) {
            ldLength = this._numberCols;
        }
        if (opts.header) {
            tr = $('<thead/>').addClass('jqplot-table-legend-header');
        } else {
            tr = $('<tr/>').addClass('jqplot-table-legend');
        }
        if (opts.reverse) {
            tr.prependTo(elem);
        }
        else {
            tr.appendTo(elem);
        }
        if (this.showSwatches) {
            if (opts.header) {
                td = $('<th/>');
                tr.append(td);
            } else {
                td = $('<td/>').addClass('jqplot-table-legend jqplot-table-legend-swatch');
                td.css({textAlign: 'center', paddingTop: rs});
                div0 = $('<div/>').addClass('jqplot-table-legend-swatch-outline');
                div1 = $('<div/>').addClass('jqplot-table-legend-swatch');
                div1.css({backgroundColor: opts.color, borderColor: opts.color});
                if (opts.color === undefined || opts.color === null) {
                    div0.css({visibility: "hidden"});
                    div1.css({visibility: "hidden"});
                }
                tr.append(td.append(div0.append(div1)));
            }
        }
        if (this.showLabels) {
            for (i = 0; i < ldLength; i++) {
                label = labelData[i];
                if (!label) {
                    label = "";
                }
                if ($.isArray(label)) {
                    dataVal = label[0];
                    label = label[1];
                    //label[0] is the data to be associated with click event
                } else {
                    dataVal = null;
                }
                if (opts.header) {
                    td = $('<th/>').addClass('jqplot-table-legend-header jqplot-table-legend-header-label');                    
                } else {
                    td = $('<td/>').addClass('jqplot-table-legend jqplot-table-legend-label');
                    if (this.clickHandler && dataVal !== null) {
                        td.addClass('jqplot-table-legend-click');
                        td.on("click", {colIdx: i, value: dataVal}, this.clickHandler);
                    }
                }
                td.css('paddingTop', rs);
                tr.append(td);
                if (this.escapeHtml) {
                    td.text(label);
                }
                else {
                    td.html(label);
                }
            }
        }
        td = null;
        div0 = null;
        div1 = null;
        tr = null;
    }
    
    //Called with scope of legend
    $.jqplot.MultiFieldLegendRenderer.prototype.draw = function(offsets, plot) {
        if (this._elem) {
            this._elem.emptyForce();
            this._elem = null;
        }

        if (this.show) {
            var series = this._series;
            var ss = 'position:absolute;';
            ss += (this.background) ? 'background:'+this.background+';' : '';
            ss += (this.border) ? 'border:'+this.border+';' : '';
            ss += (this.fontSize) ? 'font-size:'+this.fontSize+';' : '';
            ss += (this.fontFamily) ? 'font-family:'+this.fontFamily+';' : '';
            ss += (this.textColor) ? 'color:'+this.textColor+';' : '';
            ss += (this.marginTop !== undefined && this.marginTop !== null) ? 
                    'margin-top:'+this.marginTop+';' : '';
            ss += (this.marginBotton !== undefined && this.marginBottom !== null) ? 
                    'margin-bottom:'+this.marginBottom+';' : '';
            ss += (this.marginLeft !== undefined && this.marginLeft !== null) ? 
                    'margin-left:'+this.marginLeft+';' : '';
            ss += (this.marginRight !== undefined && this.marginRight !== null) ? 
                    'margin-right:'+this.marginRight+';' : '';
            // make a table.  one line label per row.
            this._elem = $('<table/>', {style: ss}).addClass('jqplot-table-legend');
            if (this.clickHandler) {
                this._elem.css('z-index', '3');
            }
            var pad = false, 
                reverse = false,
                i, s, ld,
                showCount = 0;
            this._elem.append($('<tbody/>'));
            for (i = 0; i < series.length; i++) {
                s = series[i];
                if (s.show && s.showLabel) {
                    ld = this.labels[i] || s.label.toString();
                    if (ld) {
                        showCount++;
                        var color = s.color;
                        //this does not work quite right since there may be no
                        //more series which are shown in the legend.
                        if (reverse && i < series.length - 1) {
                            pad = true;
                        }
                        else if (reverse && 
                                ((this.numberSeries === null && i == series.length - 1) ||
                                        (this.numberSeries !== null && i == this.numberSeries - 1))) {
                            pad = false;
                        }
                        addRow.call(this, {
                            labelData: ld,
                            color: color, 
                            pad: pad, 
                            reverse: reverse,
                            header: false});
                        pad = true;
                    }
                }
                ld = null;
            }
            if (this.numberSeries !== null && 
                    showCount < this.numberSeries) {
                for (i = showCount; i < this.numberSeries; i++) {
                    if (reverse && i < this.numberSeries - 1) {
                        pad = true;
                    } else {
                        pad = false;
                    }
                    addRow.call(this, {
                       pad: pad,
                       reverse: reverse,
                       header: false
                    });
                }
            }
            if (this.legendHeadings.length > 0) {
                addRow.call(this, {
                   labelData: this.legendHeadings,
                   pad: true,
                   reverse: true,
                   header: true
                });
            }

        }
        return this._elem;
    }; 
    
    /**
     * If a clickHandler has been specified, move the legend dom element to 
     * be after the event canvas.
     * Called with scope of plot.
     */
    function postDraw() {
        if (this.legend.renderer.constructor == 
            $.jqplot.MultiFieldLegendRenderer && this.legend.clickHandler){
            var e = this.legend._elem.detach();
            this.eventCanvas._elem.after(e);
        }
    }
    
})(jQuery);