//Copyright (c) InMon Corp. ALL RIGHTS RESERVED

(function($) {
    /**
     * class: $.jqplot.MultiLabelAxisRenderer
     * Renders linear y axis with up, down and central labels (or x axis with
     * left, right and central labels).
     * The up and down labels are only show if the main label is defined and
     * will be shown.
     * Uses LinearAxisRenderer to generate ticks etc.
     * 
     * Example usage:
     * plot2= $.jqplot("chart2", [s1, s2], {
     *      axes: {
     *          yaxis: {
     *               renderer: $.jqplot.MultiLabelAxisRenderer,
     *               label: "Bits/s",
     *               rendererOptions: {
     *                   upLabel: "ingress",
     *                   downLabel: "egress",
     *               }
     *               tickOptions: {
     *                   formatString: '%g', 
     *                   formatter: util.getValueFormatter(false)
     *               }
     *           }
     *       }
     * });
     * 
     */
    $.jqplot.MultiLabelAxisRenderer = function() {
        $.jqplot.LinearAxisRenderer.call(this);
     };
    
    $.jqplot.MultiLabelAxisRenderer.prototype = new $.jqplot.LinearAxisRenderer();
    $.jqplot.MultiLabelAxisRenderer.prototype.constructor = 
        $.jqplot.MultiLabelAxisRenderer;
    
    $.jqplot.MultiLabelAxisRenderer.prototype.init = function(options) {
        // prop: upLabel
        // label to be shown on the positive part of the axis (top or up for y, 
        // left for x)
        this.upLabel = null;
        // prop: downLabel
        // label to be shown on the negative part of the axis (bottom or down 
        // for y, right for x).
        this.downLabel = null;
        $.extend(true, this, {}, options);
        this.resetDataBounds();
    };
    
    // called with scope of axis
    $.jqplot.MultiLabelAxisRenderer.prototype.draw = function(ctx, plot) {
        var elem, ticks, tick, i;
        if (this.show) {
            // populate the axis labels and value properties.
            // createTicks is a method on the renderer, but
            // call it within the scope of the axis.
            this.renderer.createTicks.call(this, plot);
            // fill divs with axes labels in the right direction.
            // Need to pregenerate each axis to get its bounds and
            // position it and the labels correctly on the plot.
            // Added for theming.
            if (this._elem) {
                // Memory Leaks patch
                //this._elem.empty();
                this._elem.emptyForce();
                this._elem = null;
            }
            
            this._elem = $(document.createElement('div'));
            this._elem.addClass('jqplot-axis jqplot-'+this.name);
            this._elem.css('position', 'absolute');
            if (this.name == 'xaxis' || this.name == 'x2axis') {
                this._elem.width(this._plotDimensions.width);
            } else {
                this._elem.height(this._plotDimensions.height);
            }
            
            // create _label and upLabel, downLabel objects.
            this.labelOptions.axis = this.name;
            if (this.upLabel && this.max > 0) {
                this.upLabelOptions = 
                    $.extend(true, {}, this.labelOptions);
                this.upLabelOptions.label = this.upLabel;
                this._upLabel = new this.labelRenderer(this.upLabelOptions);
                elem = this._upLabel.draw(ctx, plot);
                elem.appendTo(this._elem);
                elem = null;
            }
            this._label = new this.labelRenderer(this.labelOptions);
            if (this._label.show) {
                elem = this._label.draw(ctx, plot);
                elem.css('white-space', 'nowrap');
                elem.appendTo(this._elem);
                elem = null;
            }
            if (this.downLabel && this.min < 0) {
                this.downLabelOptions = 
                    $.extend(true, {}, this.labelOptions);
                this.downLabelOptions.label = this.downLabel;
                this._downLabel = new this.labelRenderer(this.downLabelOptions);
                elem = this._downLabel.draw(ctx, plot);
                elem.css('white-space', 'nowrap');
                elem.appendTo(this._elem);
                elem = null;
            }
            ticks = this._ticks;
            for (i = 0; i < ticks.length; i++) {
                tick = ticks[i];
                if (tick.show && tick.showLabel && 
                        (!tick.isMinorTick || this.showMinorTicks)) {
                    this._elem.append(tick.draw(ctx, plot));
                }
            }
            tick = null;
            ticks = null;
        }
        return this._elem;
    };
    
    // called with scope of axis
    $.jqplot.MultiLabelAxisRenderer.prototype.set = function() { 
        var dim = 0,
            temp,
            w = 0,
            h = 0,
            lshow = (this._label === undefined || this._label === null) ? 
                false : this._label.show,
            ticks, tick;
        if (this.show) {
            ticks = this._ticks;
            for (var i = 0; i < ticks.length; i++) {
                tick = ticks[i];
                if (!tick._breakTick && tick.show && tick.showLabel && (!tick.isMinorTick || this.showMinorTicks)) {
                    if (this.name == 'xaxis' || this.name == 'x2axis') {
                        temp = tick._elem.outerHeight(true);
                    }
                    else {
                        temp = tick._elem.outerWidth(true);
                    }
                    if (temp > dim) {
                        dim = temp;
                    }
                }
            }
            tick = null;
            ticks = null;
            
            if (lshow) {
                w = this._label._elem.outerWidth(true);
                h = this._label._elem.outerHeight(true); 
                if (this._upLabel !== undefined && this._upLabel !== null) {
                    w = Math.max(w, this._upLabel._elem.outerWidth(true));
                    h = Math.max(h, this._upLabel._elem.outerHeight(true)); 
                }
                if (this._downLabel !== undefined && this._downLabel !== null) {
                    w = Math.max(w, this._downLabel._elem.outerWidth(true));
                    h = Math.max(h, this._downLabel._elem.outerHeight(true)); 
                }
            }
            if (this.name == 'xaxis') {
                dim = dim + h;
                this._elem.css({'height':dim+'px', left:'0px', bottom:'0px'});
            } else if (this.name == 'x2axis') {
                dim = dim + h;
                this._elem.css({'height':dim+'px', left:'0px', top:'0px'});
            } else if (this.name == 'yaxis') {
                dim = dim + w;
                this._elem.css({'width':dim+'px', left:'0px', top:'0px'});
                if (lshow && this._label.constructor == $.jqplot.AxisLabelRenderer) {
                    this._label._elem.css('width', w+'px');
                    if (this._upLabel !== undefined && this._upLabel !== null) {
                        this._upLabel._elem.css('width', w+'px');
                    }
                    if (this._downLabel !== undefined && this._downLabel !== null) {
                        this._downLabel._elem.css('width', w+'px');                        
                    }
                }
            } else {
                dim = dim + w;
                this._elem.css({'width':dim+'px', right:'0px', top:'0px'});
                if (lshow && this._label.constructor == $.jqplot.AxisLabelRenderer) {
                    this._label._elem.css('width', w+'px');
                    if (this._upLabel !== undefined && this._upLabel !== null) {
                        this._upLabel._elem.css('width', w+'px');
                    }
                    if (this._downLabel !== undefined && this._downLabel !== null) {
                        this._downLabel._elem.css('width', w+'px');                        
                    }
                }
            }
        }  
    }; 
    
    // called with scope of axis
    $.jqplot.MultiLabelAxisRenderer.prototype.pack = function(pos, offsets) {
        //we want all of the functionality of LinearAxisRenderer so call it now
        $.jqplot.LinearAxisRenderer.prototype.pack.call(this, pos, offsets);
        
        //now pack the labels
        offsets = offsets || this._offsets;
        
        var offmax = offsets.max;
        var offmin = offsets.min;
        var lshow = (this._label === undefined || this._label === null) ? 
                false : this._label.show;
        var w, h;
        
        this._offsets = offsets;
        // pixellength will be + for x axes and - for y axes because pixels 
        // are always measured from top left.
        var pixellength = offmax - offmin;
        
        if (this.show) {
            if (this.name == 'xaxis' || this.name == 'x2axis') {
                if (lshow) {
                    w = this._label._elem.outerWidth(true);
                    this._label._elem.css('left', offmin + pixellength/2 - w/2 + 'px');
                    if (this.name == 'xaxis') {
                        this._label._elem.css('bottom', '0px');
                    }
                    else {
                        this._label._elem.css('top', '0px');
                    }
                    this._label.pack();
                    if (this._upLabel !== undefined && this._upLabel !== null) {
                        w = this._upLabel._elem.outerWidth(true);
                        this._upLabel._elem.css('left', offmin + 'px');
                        if (this.name == 'xaxis') {
                            this._upLabel._elem.css('bottom', '0px');
                        }
                        else {
                            this._upLabel._elem.css('top', '0px');
                        }   
                        this._upLabel.pack();
                    }
                    if (this._downLabel !== undefined && this._downLabel !== null) {
                        w = this._downLabel._elem.outerWidth(true);
                        this._downLabel._elem.css('top', offmin + pixellength - w + 'px');
                        if (this.name == 'xaxis') {
                            this._downLabel._elem.css('bottom', '0px');
                        }
                        else {
                            this._downLabel._elem.css('top', '0px');
                        }   
                        this._downLabel.pack();
                    }
 
                }
            } else { // yaxis
                if (lshow) {
                    //align the main label with zero on the yaxis
                    h = this._label._elem.outerHeight(true);
                    if (this.min >= 0 || this.max === 0) {
                        this._label._elem.css('top', offmax - pixellength/2 - h/2 + 'px');
                    } else {
                        this._label._elem.css('top', this.u2p(0) - h/2 + 'px');
                    }
                    if (this.name == 'yaxis') {
                        this._label._elem.css('left', '0px');
                    }
                    else {
                        this._label._elem.css('right', '0px');
                    }   
                    this._label.pack();
                    if (this._upLabel !== undefined && this._upLabel !== null) {
                        h = this._upLabel._elem.outerHeight(true);
                        this._upLabel._elem.css('top', offmax  - h/2 + 'px');
                        if (this.name == 'yaxis') {
                            this._upLabel._elem.css('left', '0px');
                        }
                        else {
                            this._upLabel._elem.css('right', '0px');
                        }   
                        this._upLabel.pack();
                    }
                    if (this._downLabel !== undefined && this._downLabel !== null) {
                        h = this._downLabel._elem.outerHeight(true);
                        this._downLabel._elem.css('top', offmax - pixellength - h/2 + 'px');
                        if (this.name == 'yaxis') {
                            this._downLabel._elem.css('left', '0px');
                        }
                        else {
                            this._downLabel._elem.css('right', '0px');
                        }   
                        this._downLabel.pack();
                    }
                }
            }
        }
    };
    
})(jQuery);