/**
 * Copyright (c) InMon Corp. ALL RIGHTS RESERVED 
 */
(function($) {   
    /**
    * class: $.jqplot.TimeAxisRenderer
    * A plugin for jqPlot to render a time axis with evenly spaced intervals
    * between each tick (so that not all categories need to be labeled).
    * Must be given an array of timestamps (options.timestamps) representing
    * the time (UTC ms) for the data points in the series. The timestamps are
    * used to create ticks at regular time intervals.
    * Can also be configured with a selectedIndex of a category that is to 
    * be shown as highlighted (style configured via css for 
    * class jqplot-xaxis-tick-highlight, jqplot-x2axis-tick-highlight etc.)
    * If ticks (ie formatted time for each data point) are not set, 
    * timestamps are used for ticks, formatting with tickOptions.formatString, 
    * if given, otherwise default formatString '%Y/%m/%d'.
    * Axis options of min, max, numberTicks and tickInterval are ignored.
    * Can be configured to generate a jqplotDataClick event when the axis
    * clicked by setting the property enableClick=true. The resulting
    * event includes seriesIndex=0 and pointIndex
    * 
    * If CanvasAxisTickRenderer is used, then the number of ticks displayed
    * can be calculated based on space available and length of tick labels
    * (taking into account fonts). Otherwise the width for a tick label is 
    * hardcoded and so labels may overlap.
    * 
    * Example usage:
    * plot2 = $.jqplot("chart2", [s1, s2], {
    *           axes: {
    *                xaxis: {
    *                   renderer: $.jqplot.TimeAxisRenderer,
    *                   rendererOptions: {
    *                     timestamps: timestamps, //array of UTC timestamps (ms)
    *                     selectedIndex: selectedIndex,
    *                     enableClick: true
    *                     tickRenderer: $.jqplot.CanvasAxisTickRenderer, // only required for rotated ticks or better estimate of tick width 
    *                     tickOptions: {
    *                       fontSize: "70%", //also fontFamily, fontWeight etc.
    *                       angle: -30 //only for CanvasAxisTickRenderer
    *                     }
    *                   },
    * 
    *                   ticks: plotData.ticks, //array of strings to display for ticks
    *                   drawMajorGridlines: true
    *               },
    *       });
    * plot2.target.on('jqplotDataClick', function(evt, seriesIdx, pointIdx, data) {
    *            console.log("counters clicked on point index="+pointIdx);
    *            selectedTime = pointIdx === plotData.timestamps.length -1 ?
    *                    null : plotData.timestamps[pointIdx];
    *            //have to set selectedIndex in both options and axis
    *            plot2.options.axes.xaxis.rendererOptions.selectedIndex = pointIdx;
    *            plot2.axes.xaxis.selectedIndex = pointIdx;
    *            plot2.replot();
    *        });
    * Use css to set the cursor when the axis is clickable:
    * .jqplot-axis-clickable {
    *   cursor: crosshair
    *  }
    *  
    * Use css to set the styling of the selected tick:
    * .jqplot-xaxis-tick-highlight {
    *   top: -2px;
    *   background-color: #fff; opacity: 1; filter: alpha(opacity = 1);
    *   border: 1px solid #cccccc;
    *   border-radius: 3px;
    *   padding: 1px 1px 0px 1px;
    * } 
    */
    $.jqplot.TimeAxisRenderer = function(options) {
        $.jqplot.LinearAxisRenderer.call(this);
    };
    
    $.jqplot.TimeAxisRenderer.prototype = new $.jqplot.LinearAxisRenderer();
    $.jqplot.TimeAxisRenderer.prototype.constructor = $.jqplot.TimeAxisRenderer;
    
    /**
     * @param options
     * prop: timestamps array of timestamps (UTC ms) one for each category in 
     * the series (all series have the same categories).
     * prop: selectedIndex index of the selected timestamp whose label will be
     * be highlighted on the axis.
     * prop: enableClick true to trigger jqplotDataClick events when a tick
     * is clicked the point index corresponding to the tick is exported in the
     * event.
     */
    $.jqplot.TimeAxisRenderer.prototype.init = function(options) {
        $.extend(true, this, options);
        this._timeSteps = [1, 2, 4, 6, 12, 18];
        var db = this._dataBounds,
            axisIdx = this.name == 'xaxis' || this.name == 'x2axis' ? 0 : 1,
            i, j, series, data;
        // Go through all the series attached to this axis and find
        // the min/max bounds for this axis.
        for (i = 0; i < this._series.length; i++) {
            series = this._series[i];
            data = series.data;
            
            for (j = 0; j < data.length; j++) { 
                if (data[j][axisIdx] < db.min || db.min === null) {
                    db.min = data[j][axisIdx];
                }
                if (data[j][axisIdx] > db.max || db.max === null) {
                    db.max = data[j][axisIdx];
                }
            }
        }
    };
    
    $.jqplot.postDrawHooks.push(postDraw);
    
    /**
     * Counts the number of time categories for the given timeStep.
     * Assumes that an array of timestamps, containing one timestamp for each
     * data point (category) in the series, has been passed in as options, and  
     * that all series same set of timestamps.
     * Called with scope of axis.
     */
    $.jqplot.TimeAxisRenderer.prototype.countTimeCategories = function(step) {
        var count = 0,
            i;
        for (i = 0; i < this.timestamps.length; i++) {
            if (this.timestamps[i] % step === 0) {
                count++;
            }
        }
        return count;
    };
    
    /**
     * Creates the tick marks for the axis, using this.ticks (from options).
     * If there are a large number of data points (categories) then not all
     * categories are labeled, with evenly spaced categories being labeled.
     * Called with scope of axis.
     */
    $.jqplot.TimeAxisRenderer.prototype.createTicks = function(ctx) {
        // we're are operating on an axis here
        var userTicks = this.ticks,
            name = this.name,
            dim = (name.charAt(0) === 'x') ? 
                    this._plotDimensions.width : this._plotDimensions.height,
            threshold = 30,
            tdim = Math.max(dim, threshold+1),
            catr = this.tickRenderer === $.jqplot.CanvasAxisTickRenderer,
            maxTickWidth = 0,
            nttarget,
            timeStep = 300000, offset = 0.5, tt = 0, i, t,
            tickCount = 0,
            userTicksLength = userTicks.length,
            numTicks = userTicksLength ? userTicksLength : this.timestamps.length,
            formatString, timestamp, showLabelAndMark;
        
        this.numberTicks = numTicks;
        if (!userTicksLength) {
            formatString = this.tickOptions.formatString;
            if (!formatString) {
                formatString = '%Y/%m/%d';
            }
            this.ticks = this.timestamps;
        }
        for (i = 0; i < numTicks; i++) {
            t = new this.tickRenderer(this.tickOptions);
            if (i === this.selectedIndex) {
                t.highlight = true;
            }
            if (userTicksLength) {
                t.label = userTicks[i];
            } else {
                t.label = $.jsDate.strftime(this.timestamps[i], formatString); 
            }
            this._ticks.push(t);
            if (catr) {
                maxTickWidth = 
                    Math.max(maxTickWidth, t._textRenderer.measure(ctx, t.label)+2);
            }
        }
        
        if (!maxTickWidth) {
            //there does not seem to be a good away to calculate the tick
            //label width for other tick renderers, so just hard code something.
            maxTickWidth = 50; 
        }
        //calculate the number of ticks to display. Date labels can be long
        //If ticks are not rotated don't use too many ticks and have a high
        //spacing factor. If ticks are rotated, use a lower factor.
        if (this.tickRenderer === $.jqplot.CanvasAxisTickRenderer && 
                this.tickOptions.angle) {
            maxTickWidth = maxTickWidth -
                40 * Math.abs(Math.sin(this.tickOptions.angle/180*Math.PI));
        }
        nttarget =  Math.ceil((tdim-threshold)/maxTickWidth + 1);

        for (i = 0; i < this._timeSteps.length; i++) {
            timeStep = 300000 * this._timeSteps[i];
            tickCount = this.renderer.countTimeCategories.call(this, timeStep);
            if (tickCount > 0 && tickCount < nttarget) {
                break;
            }
        }
        
        //x values in series are numbered from 1 (not 0), 
        //ie this._series[i].data[0][xIdx] = 1
        //tick value, tt, is used to calculate tick position.
        //this.min is used to calculate gridx (see pack u2p functions).
        //For bar charts, we want the first bar butted up against the y-axis, 
        //(the first gridx should be at 0.5*barWidth) with the tick centred in 
        //the bar (aligned with gridx), so the first x - min = 0.5, min = 0.5
        //so min must be 0.5 (and not 1) and tick values (tt) must be adjusted
        //to be the same as series x values.
        //If the data point is to be plotted on the y-axis, set this.min=1
        //and similarly adjust tick value to be the same as series x.
        this.min = offset;
        this.max = numTicks + offset;
        range = this.max - this.min; 
        for (i = 0; i < numTicks; i++) {
            //adjust so that tick value is same as series x value
            //(series x values start from 1 not 0) and so tick will be
            //aligned with gridx for data point
            t = this._ticks[i];
            tt = 1 + i; 
            timestamp = this.timestamps[i];
            showLabelAndMark = i === this.selectedIndex ||
                (timestamp > 0 && timestamp % timeStep === 0);
            t.showLabel = showLabelAndMark;
            t.showMark = showLabelAndMark;
            t.showGridline = showLabelAndMark;
            t.setTick(tt, name);
        }
    };
    
    // called with scope of axis
    $.jqplot.TimeAxisRenderer.prototype.draw = function(ctx, plot) {
        var elem, i, t, tick, highlightTick = null;
        if (this.show) {
            // populate the axis label and value properties.
            // createTicks is a method on the renderer, but
            // call it within the scope of the axis.
            this.renderer.createTicks.call(this, ctx);
            // fill a div with axes labels in the right direction.
            // Need to pregenerate each axis to get its bounds and
            // position it and the labels correctly on the plot.
            // Added for theming.
            if (this._elem) {
                // this._elem.empty();
                // Memory Leaks patch
                this._elem.emptyForce();
            }

            this._elem = this._elem || 
                $('<div/>', {style: "position:absolute"}).
                    addClass("jqplot-axis jqplot-"+this.name);
            
            if (this.name == 'xaxis' || this.name == 'x2axis') {
                this._elem.width(this._plotDimensions.width);
            }
            else {
                this._elem.height(this._plotDimensions.height);
            }
            
            // create a _label object.
            this.labelOptions.axis = this.name;
            this._label = new this.labelRenderer(this.labelOptions);
            if (this._label.show) {
                elem = this._label.draw(ctx, plot);
                elem.appendTo(this._elem);
            }
    
            t = this._ticks;
            for (i = 0; i < t.length; i++) {
                tick = t[i];
                if (tick.showLabel && (!tick.isMinorTick || this.showMinorTicks)) {
                    if (tick.highlight) {
                        highlightTick = tick;
                    } else {
                        elem = tick.draw(ctx, plot);
                        elem.appendTo(this._elem);
                    }
                }
            }
            //draw the highlighted tick last so it is drawn on top
            if (highlightTick) {
                elem = highlightTick.draw(ctx, plot);
                elem.appendTo(this._elem);
            }
            if (this.enableClick) {
                this._elem.on("click", {plot: plot, axisName: this.name}, handleClick);
                this._elem.addClass("jqplot-axis-clickable");
            }
        }
        return this._elem;
    };
    
    // called with scope of axis
    $.jqplot.TimeAxisRenderer.prototype.set = function() { 
        var dim = 0;
        var temp;
        var w = 0;
        var h = 0;
        var lshow = (this._label === null) ? false : this._label.show;
        if (this.show) {
            var t = this._ticks;
            for (var i=0; i<t.length; i++) {
                var tick = t[i];
                if (tick.showLabel && (!tick.isMinorTick || this.showMinorTicks)) {
                    if (this.name == 'xaxis' || this.name == 'x2axis') {
                        temp = tick._elem.outerHeight(true);
                    } else {
                        temp = tick._elem.outerWidth(true);
                    }
                    if (temp > dim) {
                        dim = temp;
                    }
                }
            }
            
            var dim2 = 0;
            if (lshow) {
                w = this._label._elem.outerWidth(true);
                h = this._label._elem.outerHeight(true); 
            }
            if (this.name == 'xaxis') {
                dim += dim2 + h;
                this._elem.css({'height':dim+'px', left:'0px', bottom:'0px'});
            }
            else if (this.name == 'x2axis') {
                dim += dim2 + h;
                this._elem.css({'height':dim+'px', left:'0px', top:'0px'});
            }
            else if (this.name == 'yaxis') {
                dim += dim2 + w;
                this._elem.css({'width':dim+'px', left:'0px', top:'0px'});
                if (lshow && this._label.constructor == $.jqplot.AxisLabelRenderer) {
                    this._label._elem.css('width', w+'px');
                }
            }
            else {
                dim += dim2 + w;
                this._elem.css({'width':dim+'px', right:'0px', top:'0px'});
                if (lshow && this._label.constructor == $.jqplot.AxisLabelRenderer) {
                    this._label._elem.css('width', w+'px');
                }
            }
        }  
    };
    
    // called with scope of axis
    $.jqplot.TimeAxisRenderer.prototype.pack = function(pos, offsets) {
        var ticks = this._ticks;
        var max = this.max;
        var min = this.min;
        var offmax = offsets.max;
        var offmin = offsets.min;
        var lshow = (this._label === null) ? false : this._label.show;
        var i, t, shim, temp, val;
        //var labeledge;
        var pixellength, unitlength;

        for (var p in pos) {
            this._elem.css(p, pos[p]);
        }
        
        this._offsets = offsets;
        // pixellength will be + for x axes and - for y axes becasue pixels always measured from top left.
        pixellength = offmax - offmin;
        unitlength = max - min;
        
        // point to unit and unit to point conversions references to Plot DOM element top left corner.           
        this.u2p = function(u) {
            return (u - min) * pixellength / unitlength + offmin;
        };

        this.p2u = function(p) {
            return (p - offmin) * unitlength / pixellength + min;
        };
                
        if (this.name == 'xaxis' || this.name == 'x2axis'){
            this.series_u2p = function(u) {
                return (u - min) * pixellength / unitlength;
            };
            this.series_p2u = function(p) {
                return p * unitlength / pixellength + min;
            };
        } else {
            this.series_u2p = function(u) {
                return (u - max) * pixellength / unitlength;
            };
            this.series_p2u = function(p) {
                return p * unitlength / pixellength + max;
            };
        }
        
        if (this.show) {
            if (this.name == 'xaxis' || this.name == 'x2axis') {
                for (i = 0; i < ticks.length; i++) {
                    t = ticks[i];
                    if (t.show && t.showLabel) {
                        if (t.constructor == $.jqplot.CanvasAxisTickRenderer && t.angle) {
                            // will need to adjust auto positioning based on which axis this is.
                            temp = (this.name == 'xaxis') ? 1 : -1;
                            switch (t.labelPosition) {
                                case 'auto':
                                    // position at end
                                    if (temp * t.angle < 0) {
                                        shim = -t.getWidth() + t._textRenderer.height * Math.sin(-t._textRenderer.angle) / 2;
                                    } else { // position at start
                                        shim = -t._textRenderer.height * Math.sin(t._textRenderer.angle) / 2;
                                    }
                                    break;
                                case 'end':
                                    shim = -t.getWidth() + t._textRenderer.height * Math.sin(-t._textRenderer.angle) / 2;
                                    break;
                                case 'start':
                                    shim = -t._textRenderer.height * Math.sin(t._textRenderer.angle) / 2;
                                    break;
                                case 'middle':
                                    shim = -t.getWidth()/2 + t._textRenderer.height * Math.sin(-t._textRenderer.angle) / 2;
                                    break;
                                default:
                                    shim = -t.getWidth()/2 + t._textRenderer.height * Math.sin(-t._textRenderer.angle) / 2;
                                    break;
                            }
                        } else {
                            shim = -t.getWidth()/2;
                        }
                        val = this.u2p(t.value) + shim + 'px';
                        t._elem.css('left', val);
                        if (t.highlight) {
                            t._elem.addClass("jqplot-"+this.name+"-tick-highlight");
                        }
                        //apply any css font styling, font weight is not set yet
                        //in CanvasTextRenderer
                        //t._textRenderer does not exist for standard AxisTickRenderer
                        //but it is for CanvasAxisTickRenderer
                        if (t.constructor === $.jqplot.CanvasAxisTickRenderer) {
                            var styleProps = t._elem.css(["font-family", "font-size", "font-weight"]);
                            if (!t.fontFamily) {
                                t._textRenderer.fontFamily = styleProps["font-family"];
                            }
                            if (!t.fontSize)  {
                                t._textRenderer.fontSize = styleProps["font-size"];
                            }
                            if (!t.fontWeight) {
                                t._textRenderer.fontWeight = styleProps["font-weight"];
                            }
                        }
                        t.pack();
                    }
                }
                if (lshow) {
                    var w = this._label._elem.outerWidth(true);
                    this._label._elem.css('left', offmin + pixellength/2 - w/2 + 'px');
                    if (this.name == 'xaxis') {
                        this._label._elem.css('bottom', '0px');
                    }
                    else {
                        this._label._elem.css('top', '0px');
                    }
                    this._label.pack();
                }
            }
            else { //y-axis
                for (i = 0; i<ticks.length; i++) {
                    t = ticks[i];
                    if (t.show && t.showLabel) {                        
                        if (t.constructor == $.jqplot.CanvasAxisTickRenderer && t.angle) {
                            // will need to adjust auto positioning based on which axis this is.
                            temp = (this.name == 'yaxis') ? 1 : -1;
                            switch (t.labelPosition) {
                                case 'auto':
                                    // position at end
                                case 'end':
                                    if (temp * t.angle < 0) {
                                        shim = -t._textRenderer.height * Math.cos(-t._textRenderer.angle) / 2;
                                    }
                                    else {
                                        shim = -t.getHeight() + t._textRenderer.height * Math.cos(t._textRenderer.angle) / 2;
                                    }
                                    break;
                                case 'start':
                                    if (t.angle > 0) {
                                        shim = -t._textRenderer.height * Math.cos(-t._textRenderer.angle) / 2;
                                    }
                                    else {
                                        shim = -t.getHeight() + t._textRenderer.height * Math.cos(t._textRenderer.angle) / 2;
                                    }
                                    break;
                                case 'middle':
                                    shim = -t.getHeight()/2;
                                    break;
                                default:
                                    shim = -t.getHeight()/2;
                                    break;
                            }
                        }
                        else {
                            shim = -t.getHeight()/2;
                        }
                        
                        val = this.u2p(t.value) + shim + 'px';
                        t._elem.css('top', val);
                        t.pack();
                    }
                }                
                if (lshow) {
                    var h = this._label._elem.outerHeight(true);
                    this._label._elem.css('top', offmax - pixellength/2 - h/2 + 'px');
                    if (this.name == 'yaxis') {
                        this._label._elem.css('left', '0px');
                    }
                    else {
                        this._label._elem.css('right', '0px');
                    }   
                    this._label.pack();
                }                
            }
        }
    };
    
    /**
     * Triggers a jqplotDataClick event on the plot passing the value index of
     * corresponding to the tick which was clicked.
     */
    function handleClick(ev) {
        var plot = ev.data.plot,
            axis = plot.axes[ev.data.axisName],
            gridOffset = plot.eventCanvas._elem.offset(),
            x = ev.pageX - gridOffset.left, 
            tickIndex = Math.round(axis.series_p2u(x)),
            ins = [0, tickIndex-1], //x vals in series ar indexed from 1 not 0
            evt = jQuery.Event('jqplotDataClick');
        evt.which = ev.which;
        evt.pageX = ev.pageX;
        evt.pageY = ev.pageY;
        plot.target.trigger(evt, ins);
    }
    
    // called with scope of plot.
    function postDraw() {
        if (this.axes.xaxis.renderer.constructor == $.jqplot.TimeAxisRenderer &&
                this.axes.xaxis.enableClick) {
            var e = this.axes.xaxis._elem.detach();
            this.eventCanvas._elem.after(e);
        }
    }
    
})(jQuery);
