-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll env data from hosts that support the      --------
----- ENTITY-SENSORS-MIB                                       --------
-----------------------------------------------------------------------
local json = require("dkjson")
local util = require("util")
local measurement = require("measurement")

local hp_pattern = "^1%.3%.6%.1%.4%.1%.11%."
local arista_pattern = "^1%.3%.6%.1%.4%.1%.30065%."
local net_snmp_pattern = "^1%.3%.6%.1%.4%.1%.8072%."
local cisco_pattern = "^1%.3%.6%.1%.4%.1%.9%."

local entity_sensors = "1.3.6.1.2.1.99"

local es_sensor_type = entity_sensors .. ".1.1.1.1"
local es_sensor_scale = entity_sensors .. ".1.1.1.2"
local es_sensor_precision = entity_sensors .. ".1.1.1.3"
local es_sensor_value = entity_sensors .. ".1.1.1.4"
local es_sensor_status = entity_sensors .. ".1.1.1.5"
local tw_env_es = inx.TWCreate({ es_sensor_type, es_sensor_scale, es_sensor_precision, es_sensor_value, es_sensor_status })

local entPhysClass_PSU = 6
local entPhysClass_Fan = 7

local agents = {}

local function resetAgent(agent)
   agent.abort = 0
   agent.rows = 0
   agent.power = 0
   agent.temp_max = nil
   agent.fan_max = nil
   agent.humidity_max = nil

   agent.fans = {}
   agent.fan_failed = 0

   agent.psus = {}
   agent.psu_failed = 0
end

local function getAgent(ip)
   if(not agents[ip]) then
      agents[ip] = { ip=ip, entity_mtime=0 }
      resetAgent(agents[ip])
   end
   return agents[ip]
end

local function loadEntityTable(agent, entityPath)
   -- build entPhysIndex -> { entPhysClass, entPhysContainedIn }
   -- and also count the PSUs and Fans
   agent.psu_count = 0
   agent.fan_count = 0
   io.input(entityPath)
   for line in io.lines() do
      local field = 0
      local physIdx = 0
      local containedIn = 0
      local physClass = 0
      local physDescr
      for tok in string.gmatch(line, '([^,]+)') do
	 if(field == 0) then physIdx = tonumber(tok) or 0 end
	 if(field == 1) then physDescr = tok end
	 if(field == 2) then containedIn = tonumber(tok) or 0 end
	 if(field == 3) then physClass = tonumber(tok) or 0 end
	 if(field >= 3) then break end
	 field = field + 1
      end
      if(physIdx > 0 and containedIn > 0 and physClass > 0) then
	 if(agent.debug) then
	    -- inx.log(0, "physIdx=" .. tostring(physIdx) .. " class=" .. tostring(physClass) .. " containedIn=" .. tostring(containedIn))
	 end
	 agent.entityTable[physIdx] = {
	    entPhysDescr=physDescr,
	    entPhysIndex=physIdx,
	    entPhysClass=physClass,
	    entPhysContainedIn=containedIn
	 }
         if (physClass==entPhysClass_PSU) then agent.psu_count = agent.psu_count + 1
	 elseif (physClass==entPhysClass_Fan) then agent.fan_count = agent.fan_count + 1
	 end
      end
   end
end

local function parentEntityWithClass(agent, entPhysIndex, searchClass)
   local pIdx = entPhysIndex
   while (agent.entityTable[pIdx])  do
      local ent = agent.entityTable[pIdx]
      if (ent.entPhysClass == searchClass) then	return pIdx end
      pIdx = ent.entPhysContainedIn
      if(agent.debug) then
	 -- inx.log(0, "searchClass " .. tostring(searchClass) .. " not == " .. tostring(ent.entPhysClass) .. " so searching up, testing pIdx " .. tostring(pIdx))
      end
   end
   return nil
end

-- utility to get the last integer in the oid
local function oid_index(oid)
   local last
   for ss in string.gmatch(oid, "%d+") do
      last = ss
   end
   return tonumber(last)
end

local function getEntityObj(agent, oid, entPhysClass, key)
   local epIndex = oid_index(oid)
   -- if(agent.debug) then inx.log(0, "getEntityObj: " .. key .. " from sensor index " .. epIndex) end
   local objIndex = parentEntityWithClass(agent, epIndex, entPhysClass)
   if (objIndex == nil) then
     if(agent.debug) then
	-- inx.log(0, "getEntityObj: parent entity of class==" .. tostring(entPhysClass) .. " not found for: " .. tostring(epIndex))
     end
     return nil
   end
   if (not agent[key]) then agent[key] = {} end
   local lookup = agent[key]
   if (not lookup[objIndex]) then lookup[objIndex] = { entPhysIndex=objIndex } end
   return lookup[objIndex]
end

local function printAgent(agent)
   inx.log(0, "printAgent:  " .. util.table_print(agent))
end

local function computeAgentPower(agent)
   local psus = agent.psus
   local id,psu
   for id,psu in pairs(psus) do
      if(psu.power) then
	 agent.power = agent.power + psu.power
      elseif (psu.voltage and psu.current) then
	 agent.power = agent.power + (psu.voltage * psu.current)
      end
   end
end

local function walkCB(ip, status, rowid, s_type, s_scale, s_precision, s_value, s_status)
   local agent = getAgent(ip)
   if status == nil then
      agent.abort = 0
      agent.rows = agent.rows + 1
      -- see if this is for a PSU or Fan
      local psu = getEntityObj(agent, rowid, entPhysClass_PSU, "psus")
      local fan = getEntityObj(agent, rowid, entPhysClass_Fan, "fans")
      -- test the sensor_status first
      local sensor_status = tonumber(s_status) or 0
      if(sensor_status == 3) then
	 if (psu and psu.failed == nil) then
	    psu.failed = 1
	    agent.psu_failed = agent.psu_failed + 1
	 else
	    if(fan and fan.failed == nil) then
	       fan.failed = 1
	       agent.fan_failed = agent.fan_failed + 1
	    end
	 end
      elseif(sensor_status == 1) then
	 -- sensor_status is OK, so accept the reading (ignore sensor_status == 2)

	 if (psu and agent.profile == "arista") then
	    -- arista special case: the PSUs have both input and output sensors for
	    -- voltage and current, so here we ignore output sensors to ensure we
	    -- get a clean signal for input power.  Otherwise we would be multiplying
	    -- max(in_volts, out_volts) * max(in_current, out_current). Which might
	    -- be wierd.
	    local ent = agent.entityTable[psu.entPhysIndex]
	    if(ent and ent.entPhysDescr and string.find(ent.entPhysDescr, "output")) then
	       psu = nil
	    end
	 end

	 local sensor_type = tonumber(s_type)
	 local sensor_scale = 10^((9 - tonumber(s_scale)) * 3) -- e.g. 8=>milli, 7=>micro
	 local sensor_precision = 10^(tonumber(s_precision)) -- e.g. 0=>1, 1=>0.1 increments
	 local scaled_val = sensor_scale / sensor_precision * (tonumber(s_value) or 0)
	 -- now interpret sensor_type
	 if(sensor_type == 3 or sensor_type == 4) then -- volts (AC or DC)
	    if (psu) then measurement.accumulate(psu, "voltage", scaled_val, "max") end
	 elseif(sensor_type == 5) then -- Amperes
	    if (psu) then measurement.accumulate(psu, "current", scaled_val, "max") end
	 elseif(sensor_type == 6) then -- Watts
	    if (psu) then measurement.accumulate(psu, "power", scaled_val, "max") end
	 elseif(sensor_type == 8) then -- Celcius
	    measurement.accumulate(agent, "temp_max", scaled_val, "max")
	 elseif(sensor_type == 9) then -- Relative Humidity%
	    measurement.accumulate(agent, "humidity_max", scaled_val, "max")
	 elseif(sensor_type == 10) then -- fan rpm
	    if (fan) then measurement.accumulate(agent, "fan_max", scaled_val, "max") end
	 end
      end
   elseif (status == "END") then
      if not pcall(computeAgentPower, agent) then inx.log(0, "computeAgentPower failed") end

      if(agent.debug) then
         local etab = agent.entityTable
         agent.entityTable = "SUPPRESSED"
	 printAgent(agent)
	 agent.entityTable = etab
      end
      
      measurement.env(ip,
		      agent.power,
		      agent.temp_max,
		      agent.humidity_max,
		      agent.fan_max,
		      agent.fan_count,
		      agent.fan_failed,
		      agent.psu_count,
		      agent.psu_failed)
      -- clear for next time
      resetAgent(agent)
   elseif (status == "ABORT") then
      agent.abort = agent.abort + 1
   end
   return true -- keep going
end

local function readState(agent)
   local entityPath = "/usr/local/inmsf/state/agents/" .. agent.ip .. "/entity2.txt"
   local mtime = inx.modtime(entityPath)
   inx.log(1, "mtime of " .. entityPath .. " is " .. tostring(mtime))
   if not (mtime == agent.entity_mtime) then
      agent.entityTable = {}
      if not (mtime == 0) then
	 if(not pcall(loadEntityTable, agent, entityPath)) then
	    inx.log(0, "exception in loadEntityTable for " .. agent.ip)
	 end
      end
      agent.entity_mtime = mtime
   end
end

local function sendWALK(args)
   local agent = args.agent
   if(agent.debug) then inx.log(0, "EntitySensors sendWALK: " .. agent.ip) end
   local poll_mS = args.mS
   if agent.abort then
      -- it failed last time so back off
      poll_mS = poll_mS * 5
   end
   inx.timer(sendWALK, args, poll_mS)
   if(agent.up) then
      readState(agent)
      inx.TWDoTableWalk(agent.twc, walkCB)
   end
end


local function doAgent(ip, objId, up, args)
   local profile = nil
   if (string.find(objId, arista_pattern)) then profile = "arista"
   elseif (string.find(objId, hp_pattern)) then profile = "hp"
   elseif (string.find(objId, cisco_pattern)) then profile = "cisco"
   elseif (string.find(objId, net_snmp_pattern)) then profile = "net-snmp"
   end
   if (not (profile == nil)) then
      local agent = getAgent(ip)
      agent.up = up
      agent.profile = profile
      if(ip == args.debugAgent) then
	 inx.log(0, "EntitySensors doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " profile=" .. profile .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_env_es, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollEnv_EntitySensors = { agentCB = doAgent }
return pollEnv_EntitySensors
