#!/usr/bin/perl

# bgpmon - Copyright (C) 2001 David Sharp, Verio Inc.

# Permission to use, copy, and distribute this software for any
# purpose without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both that copy-
# right notice and this permission notice appear in the source. 

# This software is provided "as is" without express or implied
# warranty.

# Send comments or suggestions to:
#   djs@secure.net

# Description:  bgpmon is a tool for monitoring and recording route
# advertisments from BGP speaking routers.  It was designed to be
# be used as a tool for checking route advertisment filters and
# doing long term recording of routing data.  It operates as a peer
# of the router you intend to monitor.  Obviously then, to use this
# program you must have your router setup properly.

# When you invoke bgpmon the only two required paramaters are the
# hostname or ip address of the router and the AS number that bgpmon
# claims to represent.  There are options for setting hold time, bgp
# identifier, the verbosity and destination of the info logged.

# For doing long term recording of route data the -f and -r options
# are useful.  -f specifies a filename to log to.  If the -r option
# is used the filename is appended with the date in MMDDYY form and
# changed daily.

# Run bgpmon with no arguments to get a list of command line options.
# bgpmon has been tested on x86 BSD and HPUX using perl 4.036.  It
# should work fine on just about any other platform that has working
# perl socket code.

######################################################################

# defaults

$verbose  = 0;			# print received keepalive and open messages
$debug    = 0;			# print sent keepalive and open messages
$port     = 179;		# bgp port number
$hostid   = '127.0.0.1';	# bgp identifier
$holdtime = 180;		# bgp hold time parameter
$rotate   = 0;			# rotate logfiles daily
$thishost = '';                 # my hostname or ip address

# nhm: turn on auto-flush so we can read output through pipe better
$| = 1;

# process command line arguments and get everything setup
# for the main loop

if (!&Getopts('vdf:rp:i:t:l:')) { &usage; }
if ($#ARGV != 1) { &usage; }
$host = $ARGV[0];
$as   = $ARGV[1];
$as   =~ s/^AS//;
if ($as !~ /^\d+$/) {
  print STDERR "ERROR: bad AS number\n";
  &usage;
}
# nhm - add extra check
#if ($as < 65000) {
#    print STDERR "ERROR: AS number must be >= 65000\n";
#    &usage;
#}
if (defined($opt_p)) { $port = $opt_p;  }
if (defined($opt_r) && defined($opt_f)) { 
  $rotate  = 1;
  local(@now) = localtime(time());
  $today = sprintf(".%2.2d%2.2d%4.4d",$now[4]+1,$now[3],$now[5]+1900);
} else {
  $rotate = 0;
  $today = '';
}
if (defined($opt_f)) {
  open(SAVESTDOUT, ">&STDOUT") || die "Cant save STDOUT $!";
  close(STDOUT);
  open(STDOUT, ">>${opt_f}${today}") || die "Cant open ${opt_f}${today} $!";
  $| = 1;
}
if (defined($opt_i)) { $hostid = &ip2long($opt_i); }
else { $hostid = &ip2long($hostid); }
if (defined($opt_t) && ($opt_t =~ /^\d+$/)) { $holdtime = $opt_t; }
if (defined($opt_v)) { $verbose = 1; }
if (defined($opt_d)) { $debug = 1; }
if (defined($opt_l)) { $thishost = $opt_l; }
#
# the main loop
#

$time_of_last_keepalive = 0;
$time_of_last_keepalive_received = 0;
$negotiated_hold_time = 180;
$keepalive = 60;
$session_established = 0;

main: while (1) {

  if (!&open_session($host,$port)) {
    sleep(15);		# wait 15 sec and try to connect again
    next main;
  }

  &send_bgp_open($as,$hostid,$holdtime);

  while(1) {
    @msg = &get_bgp_message;
    if ($#msg != 2) {
      &close_session;
      sleep(15);	# wait 15 sec and try to connect again
      next main;
    }
    
    if ($rotate) {
      local(@now) = localtime(time());
      local($thisday) = sprintf(".%2.2d%2.2d%4.4d",$now[4]+1,$now[3],$now[5]+1900);
      if ($thisday ne $today) {
        $today = $thisday;
        close(STDOUT);
	open(STDOUT,">>${opt_f}${today}") || die "Open ${opt_f}${today} $!";
        $| = 1;
      }
    }

    &print_bgp_message(@msg);

    if ($session_established == 1 && (time() - $time_of_last_keepalive >= $keepalive)) {  
      &send_bgp_keepalive; 
    }

  }
}

exit;	# never reached

##########################################################3

sub usage {

print <<'EOF';

USAGE:  bgpmon [options] <host> <AS number>
  options:
            -v	         verbose logging
	    -d           very verbose logging
            -f <file>    log file
            -r           rotate logfiles daily
            -p <port>    specify port to connect to
            -i <IP>      specify bgp id to use
            -t <seconds> specify hold time
	    -l <IP>      specify address of this host

  example:
      bgpmon -f /var/log/myrouter -r 192.168.1.1 AS45453
      will peer using the given IP and AS number logging
      advertisements to /var/tmp/myrouter.YYMMDD

EOF
  exit;
}

sub open_session {

  local($router,$bgpport) = @_;

  local($AF_INET,$SOCK_STREAM,$sockaddr);
  local($proto,$localaddr,$remoteaddr);
  $t1 = &human_time;

  $AF_INET = 2;			# XXX - NON Portable
  $SOCK_STREAM = 1;		# XXX - NON Portable
  $sockaddr = 'S n a4 x8';	# XXX - NON Portable

  $proto      = (getprotobyname('tcp'))[2];
  if ($bgpport !~ /^\d+$/) {
    $bgpport  = (getservbyname($bgpport,'tcp'))[2];
}
  if($thishost eq '') {
      # it wasn't passed in, so look it up
      # if `hostname` resolves to 127.0.0.1 then
      # the bind() call will fail and you will see
      # an "invalid argument" error message. You
      # can fix this by editing /etc/hosts and
      # making a separate line for your hostname
      # with a real address.
      chop($thishost = `hostname`);
  }
  $localaddr  = (gethostbyname($thishost))[4];
  $remoteaddr = (gethostbyname($router))[4];

  $lsockaddr = pack($sockaddr, $AF_INET, 0, $localaddr);
  $rsockaddr = pack($sockaddr, $AF_INET, $bgpport, $remoteaddr);

  socket(BGPSOCK, $AF_INET, $SOCK_STREAM, $proto)	|| die $!;

  # nhm - have to set the binmode explicitly to prevent perl 5.8.0 from assuming utf8
  # this was affecting RedHat 9 installations, where perl 5.8.0 is installed by default.
  binmode(BGPSOCK, ":raw");

  bind(BGPSOCK, $lsockaddr)				|| die $!;
  if (!connect(BGPSOCK,$rsockaddr)) {
    print "$t1 FAILED CONNECT: $!\n";
    0;
    return;
  }
  1;
}

sub close_session {
  close(BGPSOCK);
  undef $buf;
  undef $bp;
  undef $bl;
  print "$t1 CLOSE\n";
  $session_established = 0;
}

sub send_bgp_open {

  local($localAS,$localbgpid,$holdtime) = @_;

  local($openmsg) = pack('A16 n C C n n N C',
	"\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377 ",
	29,		# Length
	1,		# Message type (1 = open)
	4,		# BGP version
	$localAS,	# local AS
	$holdtime,	# Holdtime
	$localbgpid,	# Local router ID
	0		# Option length
        );

  syswrite(BGPSOCK,$openmsg,29);
  if ($debug) {
    $t1 = &human_time;
    printf("%s SEND OPEN AS=%d ID=%s HT=%d\n",
      $t1,$localAS,&long2ip($localbgpid),$holdtime);
  }
}

sub send_bgp_keepalive {
    local($kamsg) = pack('A16 n C',
			 "\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377 ",
			 19,		# Message Length
			 4);		# Message Type (4 = Keepalive)
    
    # nhm: make sure we abort if this goes wrong. It's another way of
    # making sure we fail gracefully if the connection is lost
    if(syswrite(BGPSOCK,$kamsg,19) != 19) {
	die "send_bgp_keepalive: syswrite failed: $!";
    }
    $time_of_last_keepalive = time();
    if ($debug || $verbose) { 
	$t1 = &human_time;
	print "$t1 SEND KEEPALIVE\n";
    }
}

#
# Grab an input vector from the stream 
#
# nhm: modified to use select so that if the TCP connection
# is lost we don't get stuck in a blocking read

sub get_vect {  
    local($len) = @_;
    local($vect) = '';
    local($i,$rl);
    local($nfound, $timeleft, $read_in, $read_out, $exception_in, $exception_out, $select_timeout);

    while(1) {
	if ($bl - $bp >= $len) {
	    for($i=0;$i<$len;$i++) {
		vec($vect,$i,8) = vec($buf,$bp+$i,8);
	    }
	    $bp += $len;
	    last;
	}
	else {
	    for($i=0;$i<($bl-$bp);$i++) {
		vec($buf,$i,8) = vec($buf,$bp+$i,8);
	    }
	    $bp = 0;
	    $bl = $i;
	    
	    # set up the read-mask using vec(), and reuse for the exception mask
	    $read_in = '';
	    vec($read_in, fileno(BGPSOCK), 1) = 1;
	    $exception_in = $read_in;
	    
	    # loop until we get something, and keep sending keepalives if we have to wait
	    $nfound = 0;
	    while($nfound == 0) {
		# call select with timeout equal to keepalive interval? No, safer to set it to 10 seconds,
		# so that it acts as a sort of minimum timeout and avoids any danger of a busy loop
		# $select_timeout = $negotiated_hold_time;
		$select_timeout = 10;
                # print "calling select()\n";
		($nfound, $timeleft) = select($read_out=$read_in, undef, $exception_out=$exception_in, 10);
                # print "select returned nfound=$nfound, timeleft=$timeleft\n";
		if($nfound == 0) {
		    # timeout - possibly send keepalive
		    if ($session_established == 1 && (time() - $time_of_last_keepalive >= $keepalive)) {  
			&send_bgp_keepalive; 
		    }
		    # and check to see if we have heard from the peer within the negotiated holdtime
		    if($time_of_last_keepalive_received != 0 &&
                       $negotiated_hold_time != 0 &&
		       (time() - $time_of_last_keepalive_received > $negotiated_hold_time)) {
			print "seconds since last received KEEPALIVE > $negoitiated_hold_time - abort\n";
			undef;
			return;
		    }
		}
                else {
		    # something happened - check for exception
		    if(vec($exception_out, fileno(BGPSOCK), 1)) {
			print "socket exception - abort\n";
			undef;
			return;
		    }
		    # something to read?
		    if(vec($read_out, fileno(BGPSOCK), 1)) {
			$bl += ($rl = sysread(BGPSOCK,$buf,4096,$bl));
			# check for empty read - implies socket was closed
			if ($rl == 0) {
			    print "empty read - abort\n";
			    undef;
			    return;
			}
		    }
		}
	    }
	    next; 
	}
    }
    return($vect);
}

#
# separate the header and message. Check marker for all ones
# this will break if you use authentication data in the marker
#

sub get_bgp_message {  

  local($bad) = 0;
  local($header,$marker,$length,$type,$i,$message);
  $header = &get_vect(19);
  if (!defined($header)) {
    undef;
    return;
  }

  ($marker,$length,$type) = unpack("A16 n C",$header);

  for($i=0;$i<16;$i++) {
    if (vec($marker,$i,8) != 255) { $bad = 1; }
  }
  if ($bad) { 
    $t1 = &human_time;
    print "$t1 ERROR: bad marker";
    &hexdump($header,0,19);
    undef;
    return;
  }

  $message = &get_vect($length - 19);
  if (!defined($message)) {
    undef;
    return;
  }
  ($type,$length-19,$message);
}

#
# print a message
#

sub print_bgp_message {
  local($type,$length,$msg) = @_;
  local($i) = 0;

  $t1 = &human_time;
  if ($type == 1) {		#OPEN
      $session_established = 1;
      &print_bgp_open($length,$msg);
  } elsif ($type == 2) {	#UPDATE
    &print_bgp_update($length,$msg);
  } elsif ($type == 3) {	#NOTIFICATION
    print "$t1 NOTIFY:";
    $error =  unpack("\@${i}C",$msg); $i += 1;
    $error_subcode =  unpack("\@${i}C",$msg); $i += 1;
    if ($error == 1) {  #Message Header Error
      print "Message Header Error:";
    } elsif ($error == 2) { # OPEN Message Error
      print "OPEN Message Error:";
      if ($error_subcode == 1) {	# Unsupport Version number
        print "Unsupported Version Number";
      } elsif ($error_subcode == 2) {	# Bad Peer AS
        print "Bad Peer AS";
      } elsif ($error_subcode == 3) {	# Bad BGP Identifier
        print "Bad BGP Identifier";
      } elsif ($error_subcode == 4) {	# Unsupported Optional Parameter
        print "Unsupported Optional Parameter";
      } elsif ($error_subcode == 5) {	# Authentication Failure
        print "Authentication Failure";
      } elsif ($error_subcode == 6) {	# Unacceptable Hold Time
        print "Unacceptable Hold Time";
      } else {
        print "Unknown error subcode: $$error_subcode";
      }
    } elsif ($error == 3) { # UPDATE Message Error
    } elsif ($error == 4) { # Hold Timer Expiered
      print "Hold Timer Expired";
    } elsif ($error == 5) { # Finite State Machine Error
      print "Finite State Machine Error";
    } elsif ($error == 6) { # Cease
      print "Cease";
    } else {
      print "unknown notify error code: $error\n";
      &hexdump($msg,0,$length);
    }
    print "\n";
  } elsif ($type == 4) {	#KEEPALIVE
      $time_of_last_keepalive_received = time();
      if ($verbose) { print "$t1 KEEPALIVE\n"; }
  } else {
    print "$t1 ERROR: unknown message type ($type)\n";
  }
}

#
# print an update message
#

sub print_bgp_update {
  local($length,$msg) = @_;

  local($i) = 0;
  local($as, $aspath);

#
# find all the areas within the message
#

  $wlen = unpack("n",$msg);
  $i += 2;
  $wstart = $i;
  $i += $wlen;
  $palen = unpack("\@${i}n",$msg);
  $i += 2;
  $pastart = $i;
  $i += $palen;
  $nstart = $i;
  $nlen = $length - $i;

#
# print the withdrawn routes
#

  $i = $wstart;
  while($i < $wstart + $wlen) {
    $pfxlen = unpack("\@${i}C",$msg);
    $i += 1;
    if ($pfxlen > 24) {
      @prefix = unpack("\@${i}C4",$msg); $i += 4;
    } elsif ($pfxlen > 16) {
      @prefix = unpack("\@${i}C3",$msg); $i += 3;
    } elsif ($pfxlen > 8) {
      @prefix = unpack("\@${i}C2",$msg); $i += 2;
    } elsif ($pfxlen > 0) {
      @prefix = unpack("\@${i}C",$msg); $i += 1;
    } else {
      @prefix = (); 
    }

   # $$$ nhm $$$ - colon delimited output (no timestamp)
   # <+/->:<CIDR>:<NextHop>:<ifIndex>:<Protocol>:<ASPath>:<Communities>:<LocalPref>
   # was:
   # printf("$t1 - %d.%d.%d.%d/%d\n",
   # $prefix[0],$prefix[1],$prefix[2],$prefix[3],$pfxlen);
   printf("-,%d.%d.%d.%d/%d,,,,,,\n",
	  $prefix[0],$prefix[1],$prefix[2],$prefix[3],$pfxlen);
  }

#
# variables and constants used in path attributes
#

  $PAF_OPTIONAL = 0x80;
  $PAF_TRANSITIVE = 0x40;
  $PAF_PARTIAL = 0x20;
  $PAF_EXTLENGTH = 0x10;

  $origin = '';
  $aspath = '';
  $nexthop = '';
  $multiexitdisc = '';
  $localpref = '';
  $atomicaggregate = '';
  $aggregatoras = '';
  $aggregator = '';
  $communities = '';

#
# parse out the path attributes.  note: only the stuff listed
# in RFC 1771 is done here.  The community attribute might be
# useful for someone.
#

  $i = $pastart;
  while($i < $pastart + $palen) {
    ($paflags,$patype) = unpack("\@${i}CC",$msg); $i += 2;
    if ($paflags & $PAF_EXTLENGTH) {
       $alength = unpack("\@${i}n",$msg); $i += 2;
    } else {
       $alength = unpack("\@${i}C",$msg); $i += 1;
    }
    if ($patype == 1) {		#ORIGIN
       $origin = unpack("\@${i}C",$msg); $i += 1;
    } elsif ($patype == 2) {	#ASPATH
      $aspathstart = $i;
      $aspathlen   = $alength;
      $aspath = '';
      while($i < $aspathstart + $aspathlen) {
        ($pathstype,$pathslen) = unpack("\@${i}CC",$msg); $i += 2;
	if ($pathstype == 1) {		#AS_SET
          $aspath .= " [ ";
          for($j=0;$j<$pathslen;$j++) {
            $as = unpack("\@${i}n",$msg); $i += 2;
	    $aspath .= "$as ";
	  }
          $aspath .= "] ";
	} elsif ($pathstype == 2) {	#AS_SEQUENCE
          for($j=0;$j<$pathslen;$j++) {
            $as = unpack("\@${i}n",$msg); $i += 2;
	    $aspath .= "$as ";
	  }
	} elsif ($pathstype == 3) {	#AS CONFED SEQUENCE
          $aspath .= "( ";
          for($j=0;$j<$pathslen;$j++) {
            $as = unpack("\@${i}n",$msg); $i += 2;
	    $aspath .= "$as ";
	  }
          $aspath .= ") ";
	} elsif ($pathstype == 4) {	#AS CONFED SET
	  if($verbose) { # nhm
	    print "AS CONFED SET len=$pathslen (i=$i, aspathstart=$aspathstart, aspathlen=$aspathlen)\n";
          }
          $aspath .= "([ ";
          for($j=0;$j<$pathslen;$j++) {
            $as = unpack("\@${i}n",$msg); $i += 2;
	    $aspath .= "$as ";
	  }
          $aspath .= "]) ";
	} else {
	  if($verbose) { # nhm
	    print "unknown path type ($pathstype)";
	    &hexdump($msg,$i,$pathslen);
	  }
	  $i = $aspathstart + $aspathlen; # nhm - skip to the end
	}

      }
    } elsif ($patype == 3) {	#NEXT_HOP
       $nexthop = unpack("\@${i}N",$msg); $i += 4;
    } elsif ($patype == 4) {	#MULTI_EXIT_DISC
       $multiexitdisc = unpack("\@${i}N",$msg); $i += 4;
    } elsif ($patype == 5) {	#LOCAL_PREF
       $localpref = unpack("\@${i}N",$msg); $i += 4;
    } elsif ($patype == 6) {	#ATOMIC_AGGREGATE
       $atomicaggregate = 1;
    } elsif ($patype == 7) {	#AGGREGATOR
#       ($aggregatoras,$aggregator) = unpack("\@${i}nN",$msg); $i += 6;
       ($aggregatoras,$ia,$ib,$ic,$id) = unpack("\@${i}nCCCC",$msg); $i += 6;
       $aggregator = "$ia.$ib.$ic.$id";
    } elsif ($patype == 8) {	#COMMUNITIES
	# list of 4-byte numbers, so $alength should be multiple of 4
	# 0x00000000 thru 0x0000FFFF are reserved (i.e. 0:*)
	# 0xFFFF0000 thru 0xFFFFFFFF are reserved (i.e. 65535:*)
	# by convention, the high order 16 bits = AS number of
	# entity who set the value.
	# well known values are:
	# 0xFFFFFF01 = NO-EXPORT
	# 0xFFFFFF02 = NO-ADVERTISE
	# 0xFFFFFF03 = NO-EXPORT-SUBCONFED
	$commstart = $i;
	$commlen = $alength;
	$communities = '';
	while($i < $commstart + $commlen) {
	    ($comm) = unpack("\@${i}N",$msg); $i += 4;
	    $communities .= "$comm ";
	}
	
    } else {
      if($verbose) { # nhm
        print "unknown attribute type ($patype)";
        &hexdump($msg,$i,$alength);
      }
      $i += $alength;  # skip over the whole attribute
    }
  }

  # remove leading and trailing spaces
  $aspath =~ s/^\s+//;
  $aspath =~ s/\s+$//;

#
#  Print the advertised networks with the AS path, origin,
#  and agregator info if applicable.
#

  $i = $nstart;
  while($i < $nstart + $nlen) {
    $pfxlen = unpack("\@${i}C",$msg);
    $i += 1;
    if ($pfxlen > 24) {
      @prefix = unpack("\@${i}C4",$msg); $i += 4;
    } elsif ($pfxlen > 16) {
      @prefix = unpack("\@${i}C3",$msg); $i += 3;
    } elsif ($pfxlen > 8) {
      @prefix = unpack("\@${i}C2",$msg); $i += 2;
    } elsif ($pfxlen > 0) {
      @prefix = unpack("\@${i}C",$msg); $i += 1;
    } else {
      @prefix = (); 
    }

    # $$$ nhm $$$ - colon separated output (no timestamp)
    # <+/->:<CIDR>:<NextHop>:<ifIndex>:<Protocol>:<ASPath>:<Communities>:<LocalPref>
    # was:
    # printf("$t1 + %d.%d.%d.%d/%d $aspath",
    #        $prefix[0],$prefix[1],$prefix[2],$prefix[3],$pfxlen);
    # if ($origin == 0) { print " IGP"; } 
    # elsif ($origin == 1) { print " EGP"; }
    # else { print " i"; }
    # if ($aggregator) { printf(" A=(%d,%8.8x)",$aggregatoras,$aggregator); }
    # if ($aggregator) { printf(" A=(%d,%s)",$aggregatoras,$aggregator); }
    # print "\n";

    $prot = "i";
    if ($origin == 0) { $prot  = "IGP"; } 
    elsif ($origin == 1) { $prot = "EGP"; }
    
    printf("+,%d.%d.%d.%d/%d,,,%s,%s,%s,%s\n",
	   $prefix[0], $prefix[1], $prefix[2], $prefix[3], $pfxlen, $prot, $aspath, $communities, $localpref);
  }
}

#
# print an open message.  note: options are not parsed, but
# simply dumped in hex format.
#

sub print_bgp_open {
  local($length,$msg) = @_;
  local($ver,$as,$peerholdtime,$id,$optlen);
  local($i) = 0;

  if ($length < 10) {
    print "\n$t1 OPEN ERROR: truncated message (length = $length)\n";
    &hexdump($msg,0,$length);
    return;
  }
  ($ver,$as,$peerholdtime,$id,$optlen) = unpack("CnnNC",$msg);
  printf("\n$t1 OPEN Ver=%d AS=%d Holdtime=%d ID=%s",$ver,$as,$peerholdtime,&long2ip($id));
  if ($optlen) {
    printf("\n$t1 OPEN Options (len=%d):",$optlen); 
    $i = 10;
    &xdump($msg,$i,$optlen);
    while ($i < $length) {
      $opcode = unpack("\@${i}C",$msg); $i += 1;
      $oplen  = unpack("\@${i}C",$msg); $i += 1;
      printf("\n$t1 OPEN Option 0x%2.2x",$opcode);
      if ($oplen > 0) {
        print " DATA:";
        &xdump($msg,$i,$oplen); $i += $oplen;
      }
    }
  } 

  $negotiated_hold_time = $holdtime;
  if ($peerholdtime < $holdtime) { $negotiated_hold_time = $peerholdtime; }
  $keepalive = $negotiated_hold_time / 3;

  if ($debug ) {
    printf("\nOPEN: peerholdtime=%d holdtime=%d negotiated_hold_time=%d keepalive=%d\n",
            $peerholdtime, $holdtime, $negotiated_hold_time, $keepalive);
  }

  print "\n";
}

#
# various utility subroutines
#

sub hexdump {
  local($data,$offset,$length) = @_;
  local($i,$len);

  print "hexdump(len=$length):\n";
  $len = $length;
  if($len > 128) {
	# truncate the dump
	$len = 128;
  }

  for ($i=0;$i<$len;$i++) {
    if (($i % 16) == 0) { printf("\n%4.4x: ",$i); }
    printf(" %2.2x",vec($data,$offset + $i,8));
  }
  print "\n";
}

sub xdump {
  local($data,$offset,$length) = @_;
  local($i);

  for ($i=0;$i<$length;$i++) {
    printf(" %2.2x",vec($data,$offset + $i,8));
  }
}


sub human_time {
  local(@now);
  @now = localtime(time());
  sprintf("%2.2d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",
           $now[4]+1,$now[3],$now[5],$now[2],$now[1],$now[0]);
}

sub ip2long {
  local(@ip);
  if (@ip = ($_[0] =~ /\s*(\d+)\.(\d+)\.(\d+)\.(\d+)\s*/)) {
    return(unpack("N",pack("CCCC",@ip)));
  }
  undef;
}

sub long2ip {
  sprintf("%d.%d.%d.%d",unpack("CCCC",pack("N",$_[0])));
}


#
# This was lifted out of the perl distribution.  I have no idea
# who actually wrote it.
#

# getopts.pl - a better getopt.pl

# Usage:
#      do Getopts('a:bc');  # -a takes arg. -b & -c not. Sets opt_* as a
#                           #  side effect.

sub Getopts {
    local($argumentative) = @_;
    local(@args,$_,$first,$rest);
    local($errs) = 0;
    local($[) = 0;

    @args = split( / */, $argumentative );
    while(@ARGV && ($_ = $ARGV[0]) =~ /^-(.)(.*)/) {
        ($first,$rest) = ($1,$2);
        $pos = index($argumentative,$first);
        if($pos >= $[) {
            if($args[$pos+1] eq ':') {
                shift(@ARGV);
                if($rest eq '') {
                    ++$errs unless @ARGV;
                    $rest = shift(@ARGV);
                }
                eval "\$opt_$first = \$rest;";
            }
            else {
                eval "\$opt_$first = 1";
                if($rest eq '') {
                    shift(@ARGV);
                }
                else {
                    $ARGV[0] = "-$rest";
                }
            }
        }
        else {
            print STDERR "Unknown option: $first\n";
            ++$errs;
            if($rest ne '') {
                $ARGV[0] = "-$rest";
            }
            else {
                shift(@ARGV);
            }
        }
    }
    $errs == 0;
}

